/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

package im.netease.nim;

import android.content.Context;
import android.text.TextUtils;

import com.netease.nimlib.sdk.NIMClient;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;

import androidx.annotation.NonNull;
import im.netease.nim.auth.AuthBridge;
import im.netease.nim.base.Handler;
import im.netease.nim.chatroom.ChatRoomMsgBridge;
import im.netease.nim.msg.MsgBridge;
import im.netease.nim.nos.NosBridge;
import im.netease.nim.passthrough.PassthroughBridge;
import im.netease.nim.signalling.SignallingBridge;
import io.flutter.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * NimPlugin
 */
public class NimPlugin implements FlutterPlugin, MethodCallHandler {

    public static final String TAG = "NimPlugin";

    public static final String PLUGIN_NAME = "nim_plugin";

    private MethodChannel channel;

    private Context context;

    private Map<String, Handler> handlerMap;

    private AtomicBoolean config = new AtomicBoolean(false);

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        config.set(false);
        channel = new MethodChannel(binding.getBinaryMessenger(), PLUGIN_NAME);
        channel.setMethodCallHandler(this);
        context = binding.getApplicationContext();
        initBridge();
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        if (channel != null) {
            channel.setMethodCallHandler(null);
            channel = null;
        }
        if (handlerMap != null) {
            for (Map.Entry<String, Handler> entry : handlerMap.entrySet()) {
                entry.getValue().unInit();
            }
            handlerMap.clear();
            handlerMap = null;
        }
    }

    private void initBridge() {
        handlerMap = new HashMap<>();
        new AuthBridge(channel, context).register(handlerMap);
        new SignallingBridge(channel, context).register(handlerMap);
        new PassthroughBridge(channel, context).register(handlerMap);
        new MsgBridge(channel, context).register(handlerMap);
        new NosBridge(channel, context).register(handlerMap);
        new ChatRoomMsgBridge(channel, context).register(handlerMap);
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        String moduleName = call.argument("module");
        String methodName = call.method;
        Log.i(TAG, "moduleName=" + moduleName + " methodName=" + methodName);
        if (!TextUtils.isEmpty(moduleName)) {
            Handler handler = handlerMap.get(moduleName);
            if (handler != null) {
                handler.handle(methodName, call, result);
            } else {
                result.success(null);
            }
        } else if (call.method.equals("config")) {
            if (config.compareAndSet(false, true)) {/// avoid duplicate init
                NIMClient.initSDK();
                postInit();
            }
            result.success(null);
        } else {
            result.notImplemented();
        }
    }

    /**
     * post init
     */
    private void postInit() {
        for (Map.Entry<String, Handler> entry : handlerMap.entrySet()) {
            entry.getValue().init();
        }
    }

}
