/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

package im.netease.nim.auth;

import android.content.Context;
import android.text.TextUtils;

import com.netease.nimlib.sdk.NIMClient;
import com.netease.nimlib.sdk.Observer;
import com.netease.nimlib.sdk.RequestCallbackWrapper;
import com.netease.nimlib.sdk.ResponseCode;
import com.netease.nimlib.sdk.StatusCode;
import com.netease.nimlib.sdk.auth.AuthService;
import com.netease.nimlib.sdk.auth.AuthServiceObserver;
import com.netease.nimlib.sdk.auth.LoginInfo;
import com.netease.nimlib.sdk.auth.OnlineClient;
import com.netease.nimlib.sdk.msg.MessageBuilder;
import com.netease.nimlib.sdk.msg.constant.SessionTypeEnum;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import im.netease.nim.base.Handler;
import im.netease.nim.util.StatusCodeUtil;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

/**
 * Created by hzsunyj on 2019-08-09.
 */
public class AuthBridge extends Handler {

    public AuthBridge(MethodChannel channel, Context context) {
        super(channel, context);
    }

    private AuthService service;

    private AuthServiceObserver serviceObserver;

    private LoginInfo loginInfo;

    private List<OnlineClient> onlineClients;

    Observer<StatusCode> observer = new Observer<StatusCode>() {

        @Override
        public void onEvent(StatusCode statusCode) {
            if (statusCode == null) {
                return;
            }
            if (statusCode == StatusCode.SYNCING) {
                // call syncing
            } else {
                MapBuilder map = map();
                map.put("module", observerModuleName());
                map.put("statusCode", StatusCodeUtil.convertStatusCode(statusCode));
                channel.invokeMethod("onOnlineStatus", map.build());
            }
        }
    };

    @Override
    public void unInit() {
        if (serviceObserver != null) {
            serviceObserver.observeOnlineStatus(observer, false);
        }
        service = null;
        serviceObserver = null;
    }

    @Override
    public void postInit() {
        service = NIMClient.getService(AuthService.class);
        serviceObserver = NIMClient.getService(AuthServiceObserver.class);
    }

    @Override
    public void registerObserver() {
        serviceObserver.observeOnlineStatus(observer, true);
    }

    @Override
    public String observerModuleName() {
        return "AuthObserver";
    }

    @Override
    public String moduleName() {
        return "AuthService";
    }

    public List<OnlineClient> getOnlineClients() {
        return onlineClients;
    }

    public void setOnlineClients(List<OnlineClient> onlineClients) {
        this.onlineClients = onlineClients;
    }

    @Override
    public void handle(String method, final MethodCall call, MethodChannel.Result result) {
        if (TextUtils.isEmpty(method)) {
            return;
        }
        switch (method) {
            case "login":
                String account = call.argument("account");
                String token = call.argument("token");
                String appKey = call.argument("appKey");
                loginInfo = new LoginInfo(account, token, appKey);
                service.login(loginInfo).setCallback(new RequestCallbackWrapper() {

                    @Override
                    public void onResult(int code, Object result, Throwable exception) {
                        final MapBuilder builder = Handler.buildArg(getReceiptId(call), moduleName());
                        builder.put("code", code);
                        channel.invokeMethod("onLogin", builder.build());
                    }
                });
                result.success(null);
                break;
            case "logout":
                service.logout();
                result.success(null);
                break;
            case "getCurrentAccount":
                result.success(loginInfo != null ? loginInfo.getAccount() : "");
                break;
            case "getLoginInfo":
                Map<String,String> loginInfoMap = new HashMap<>();
                //没有接口获取当前登录的账号，只能build一个message后获取
                String currentLoginAccount = MessageBuilder.createTextMessage("", SessionTypeEnum.None, "").getFromAccount();
                loginInfoMap.put("account", currentLoginAccount);
                //loginInfoMap.put("appKey", loginInfo.getAppKey());
                //loginInfoMap.put("token", loginInfo.getToken());
                result.success(loginInfoMap);
                break;
            case "kickOtherClient":
                OnlineClient client = null;
                String deviceId = call.argument("deviceId");
                if (onlineClients != null && onlineClients.size() > 0) {
                    for (OnlineClient onlineClient : onlineClients) {
                        if (TextUtils.equals(onlineClient.getClientIp(), deviceId)) {
                            client = onlineClient;
                            break;
                        }
                    }
                }
                final MapBuilder builder = Handler.buildArg(getReceiptId(call), moduleName());
                if (client == null) {
                    builder.put("code", ResponseCode.RES_EPARAM);
                    channel.invokeMethod("onKickOtherClient", builder.build());
                } else {
                    service.kickOtherClient(client).setCallback(new RequestCallbackWrapper<Void>() {

                        @Override
                        public void onResult(int code, Void result, Throwable exception) {
                            builder.put("code", code);
                            channel.invokeMethod("onKickOtherClient", builder.build());
                        }
                    });
                }
                result.success(null);
                break;
        }
    }

}
