/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

package im.netease.nim.chatroom;

import android.content.Context;
import android.text.TextUtils;
import android.util.Log;

import com.netease.nimlib.sdk.NIMClient;
import com.netease.nimlib.sdk.Observer;
import com.netease.nimlib.sdk.RequestCallbackWrapper;
import com.netease.nimlib.sdk.ResponseCode;
import com.netease.nimlib.sdk.chatroom.ChatRoomService;
import com.netease.nimlib.sdk.chatroom.ChatRoomServiceObserver;
import com.netease.nimlib.sdk.chatroom.constant.MemberQueryType;
import com.netease.nimlib.sdk.chatroom.constant.MemberType;
import com.netease.nimlib.sdk.chatroom.model.ChatRoomIndependentCallback;
import com.netease.nimlib.sdk.chatroom.model.ChatRoomInfo;
import com.netease.nimlib.sdk.chatroom.model.ChatRoomMember;
import com.netease.nimlib.sdk.chatroom.model.ChatRoomMemberUpdate;
import com.netease.nimlib.sdk.chatroom.model.ChatRoomMessage;
import com.netease.nimlib.sdk.chatroom.model.ChatRoomMessageExtension;
import com.netease.nimlib.sdk.chatroom.model.CustomChatRoomMessageConfig;
import com.netease.nimlib.sdk.chatroom.model.EnterChatRoomData;
import com.netease.nimlib.sdk.chatroom.model.EnterChatRoomResultData;
import com.netease.nimlib.sdk.msg.constant.MsgTypeEnum;
import com.netease.nimlib.sdk.msg.model.QueryDirectionEnum;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import im.netease.nim.base.Handler;
import im.netease.nim.helper.ChatRoomMessageHelper;
import im.netease.nim.util.MemberTypeUtil;
import im.netease.nim.util.MsgTypeUtil;
import im.netease.nim.util.StatusCodeUtil;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

/**
 * Created by hzsunyj on 2020/6/18.
 */
public class ChatRoomMsgBridge extends Handler {

    public ChatRoomMsgBridge(MethodChannel channel, Context context) {
        super(channel, context);
    }

    ChatRoomService msgService;

    ChatRoomServiceObserver msgServiceObserve;

    Observer<List<ChatRoomMessage>> observer = new Observer<List<ChatRoomMessage>>() {

        @Override
        public void onEvent(List<ChatRoomMessage> imMessages) {
            onReceiveMessage(imMessages);
        }
    };

    @Override
    public void unInit() {
        if (msgServiceObserve != null) {
            msgServiceObserve.observeReceiveMessage(observer, false);
        }
        msgService = null;
        msgServiceObserve = null;
    }

    @Override
    public void postInit() {
        msgService = NIMClient.getService(ChatRoomService.class);
        msgServiceObserve = NIMClient.getService(ChatRoomServiceObserver.class);
    }

    @Override
    public void registerObserver() {
        msgServiceObserve.observeReceiveMessage(observer, true);
    }

    private void onReceiveMessage(List<ChatRoomMessage> imMessages) {
        MapBuilder builder = buildArg(observerModuleName());
        ArrayList<Map<String, Object>> arrayList = new ArrayList<>(imMessages.size());
        for (ChatRoomMessage message : imMessages) {
            if (message == null) {// ignore
                continue;
            }
            MapBuilder data = map();
            buildMessage(message, data);
            arrayList.add(data.build());
        }
        builder.put("messages", arrayList);
        channel.invokeMethod("onReceiveMessage", builder.build());
    }

    @Override
    public String moduleName() {
        return "ChatRoomService";
    }

    @Override
    public String observerModuleName() {
        return "ChatRoomObserver";
    }

    @Override
    public void handle(String method, MethodCall call, MethodChannel.Result result) {
        if (TextUtils.isEmpty(method)) {
            return;
        }
        switch (method) {
            case "enterChatRoom":
                enterChatRoom(call);
                result.success(null);
                break;
            case "getEnterErrorCode":
                String roomId = call.argument("roomId");
                result.success(msgService.getEnterErrorCode(roomId));
                break;
            case "exitChatRoom":
                exitChatRoom(call);
                result.success(null);
                break;
            case "sendMessage":
                onSendMessage(call);
                result.success(null);
                break;
            case "downloadAttachment":
                downloadAttachment(call);
                result.success(null);
                break;
            case "pullMessageHistory":
                pullMessageHistory(call);
                result.success(null);
                break;
            case "fetchRoomInfo":
                fetchRoomInfo(call);
                result.success(null);
                break;
            case "fetchRoomMembers":
                fetchRoomMembers(call);
                result.success(null);
                break;
            case "updateMyRoomRole":
                updateMyRoomRole(call);
                result.success(null);
                break;
        }
    }


    private void buildMember(ChatRoomMember member, MapBuilder data) {
        data.put("roomId", member.getRoomId());
        data.put("account", member.getAccount());
        data.put("memberType", member.getMemberType().getValue());
        data.put("memberLevel", member.getMemberLevel());
        data.put("nick", member.getNick());
        data.put("avatar", member.getAvatar());
        data.put("extension", member.getExtension());
        data.put("online", member.isOnline());
        data.put("inBlackList", member.isInBlackList());
        data.put("isMuted", member.isMuted());
        data.put("isValid", member.isValid());
        data.put("enterTime", member.getEnterTime());
        data.put("updateTime", member.getUpdateTime());
        data.put("isTempMuted", member.isTempMuted());
        data.put("tempMuteDuration", member.getTempMuteDuration());
    }

    private void fetchRoomMembers(final MethodCall call) {
        String roomId = call.argument("roomId");
        Integer queryType = call.argument("type");
        Map member = call.argument("member");
        Integer limit = call.argument("limit");
        Integer time = 0;
        Integer memberType = member != null ? (Integer) member.get("memberType") : null;
        if (memberType != null) {
            if (memberType == MemberType.GUEST.getValue() || memberType == MemberType.ANONYMOUS.getValue()) {
                time = member != null ? (Integer) member.get("enterTime") : null;
            } else {
                time = member != null ? (Integer) member.get("updateTime") : null;
            }

        }
        msgService.fetchRoomMembers(roomId, MemberQueryType
                                            .typeOfValue(queryType == null ? MemberQueryType.UNKNOWN.getValue() : queryType),
                                    time == null ? 0 : time, limit == null ? 20 : limit).setCallback(
                new RequestCallbackWrapper<List<ChatRoomMember>>() {

                    @Override
                    public void onResult(int code, List<ChatRoomMember> result, Throwable exception) {
                        MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                        ArrayList<Map<String, Object>> arrayList = new ArrayList<>(result.size());
                        for (ChatRoomMember roomMember : result) {
                            if (roomMember == null) {
                                continue;
                            }
                            MapBuilder data = map();
                            buildMember(roomMember, data);
                            arrayList.add(data.build());
                        }
                        builder.put("members", arrayList);
                        channel.invokeMethod("onFetchRoomMembers", builder.build());
                    }
                });
    }

    private void updateMyRoomRole(final MethodCall call) {
        String roomId = call.argument("roomId");
        Map memberUpdate = call.argument("memberUpdate");
        ChatRoomMemberUpdate update = new ChatRoomMemberUpdate();
        if (memberUpdate != null) {
            update.setNick((String) memberUpdate.get("nick"));
            update.setAvatar((String) memberUpdate.get("avatar"));
            update.setExtension((Map) memberUpdate.get("extension"));
            update.setNeedSave((Boolean) memberUpdate.get("needSave"));
        }
        boolean needNotify = call.argument("needNotify");
        Map notifyExtension = call.argument("notifyExtension");
        msgService.updateMyRoomRole(roomId, update, needNotify, notifyExtension).setCallback(
                new RequestCallbackWrapper() {

                    @Override
                    public void onResult(int code, Object result, Throwable exception) {
                        MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                        builder.put("code", code);
                        channel.invokeMethod("onUpdateMyRoomRole", builder.build());
                    }
                });
    }

    private void pullMessageHistory(MethodCall call) {
        String roomId = call.argument("roomId");
        Number startTime = call.argument("startTime");
        Integer limit = call.argument("limit");
        Boolean queryNew = call.argument("queryNew");
        List<Integer> types = call.argument("types");
        MsgTypeEnum[] typeEnums = MsgTypeUtil.listMapMsgType(types);
        msgService.pullMessageHistoryExType(roomId, startTime.longValue(), limit == null ? 20 : limit,
                                            queryNew != null &&
                                            queryNew ? QueryDirectionEnum.QUERY_NEW : QueryDirectionEnum.QUERY_OLD,
                                            typeEnums).setCallback(new RequestCallbackWrapper<List<ChatRoomMessage>>() {

            @Override
            public void onResult(int code, List<ChatRoomMessage> result, Throwable exception) {
                MapBuilder builder = buildArg(moduleName());
                ArrayList<Map<String, Object>> arrayList = new ArrayList<>(result.size());
                for (ChatRoomMessage message : result) {
                    if (message == null) {
                        continue;
                    }
                    MapBuilder data = map();
                    buildMessage(message, data);
                    arrayList.add(data.build());
                }
                builder.put("messages", arrayList);
                channel.invokeMethod("onPullMessageHistory", builder.build());
            }
        });
    }

    private void buildMessage(ChatRoomMessage message, MapBuilder data) {
        data.put("uuid", message.getUuid());
        data.put("sessionId", message.getSessionId());
        data.put("sessionType", message.getSessionType().getValue());
        data.put("attach", message.getAttachStr());
        data.put("time", message.getTime());
        data.put("serverId", message.getServerId());
        data.put("msgType", message.getMsgType().getValue());
        data.put("content", message.getContent());
        data.put("fromAccount", message.getFromAccount());
        data.put("msgStatus", message.getStatus().getValue());
        data.put("msgDirection", message.getDirect().getValue());
        ChatRoomMessageExtension extension = message.getChatRoomMessageExtension();
        if (extension != null) {
            MapBuilder messageExtension = map();
            messageExtension.put("roleInfoTimeTag", extension.getRoleInfoTimeTag());
            messageExtension.put("nick", extension.getSenderNick());
            messageExtension.put("avatar", extension.getSenderAvatar());
            messageExtension.put("senderExtension", extension.getSenderExtension());
            data.put("messageExtension", messageExtension.build());
        }
        CustomChatRoomMessageConfig config = message.getChatRoomConfig();
        if (config != null) {
            MapBuilder configMap = map();
            configMap.put("skipHistory", config.skipHistory);
            data.put("roomMessageConfig", configMap.build());
        }
    }

    private void fetchRoomInfo(final MethodCall call) {
        String roomId = call.argument("roomId");
        msgService.fetchRoomInfo(roomId).setCallback(new RequestCallbackWrapper<ChatRoomInfo>() {

            @Override
            public void onResult(int code, ChatRoomInfo result, Throwable exception) {
                MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                builder.put("code", code);
                builder.put("roomInfo", buildChatRoomInfo(result));
                channel.invokeMethod("onFetchRoomInfo", builder.build());
            }
        });
    }

    private void downloadAttachment(final MethodCall call) {
        Map map = call.argument("message");
        if (map == null) {
            MapBuilder builder = buildArg(getReceiptId(call), moduleName());
            builder.put("code", ResponseCode.RES_EPARAM);
            channel.invokeMethod("onDownloadAttachment", builder.build());
        } else {
            boolean thumb = call.argument("thumb");
            ChatRoomMessage message = ChatRoomMessageHelper.convert(map);
            if (message == null) {
                MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                builder.put("code", ResponseCode.RES_EPARAM);
                channel.invokeMethod("onDownloadAttachment", builder.build());
                return;
            }
            msgService.downloadAttachment(message, thumb).setCallback(new RequestCallbackWrapper<Void>() {

                @Override
                public void onResult(int i, Void aVoid, Throwable throwable) {
                    MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                    builder.put("code", i);
                    channel.invokeMethod("onDownloadAttachment", builder.build());
                }
            });
        }
    }

    private void onSendMessage(final MethodCall call) {
        Map map = call.argument("message");
        if (map == null) {
            MapBuilder builder = buildArg(getReceiptId(call), moduleName());
            builder.put("code", ResponseCode.RES_EPARAM);
            channel.invokeMethod("onSendMessage", builder.build());
        } else {
            boolean resend = call.argument("resend");
            ChatRoomMessage message = ChatRoomMessageHelper.convert(map);
            if (message == null) {
                MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                builder.put("code", ResponseCode.RES_EPARAM);
                channel.invokeMethod("onSendMessage", builder.build());
                return;
            }
            Log.d("ChatRoom", "send" + message.getContent());
            msgService.sendMessage(message, resend).setCallback(new RequestCallbackWrapper<Void>() {

                @Override
                public void onResult(int i, Void aVoid, Throwable throwable) {
                    MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                    builder.put("code", i);
                    channel.invokeMethod("onSendMessage", builder.build());
                }
            });
        }
    }

    private void exitChatRoom(final MethodCall call) {
        String roomId = call.argument("roomId");
        msgService.exitChatRoom(roomId);
    }

    // 进入聊天室
    private void enterChatRoom(final MethodCall call) {
        Map data = call.argument("data");
        String roomId = data != null ? (String) data.get("roomId") : null;
        if (TextUtils.isEmpty(roomId)) {
            MapBuilder builder = buildArg(getReceiptId(call), moduleName());
            builder.put("code", ResponseCode.RES_EPARAM);
            channel.invokeMethod("onEnterChatRoom", builder.build());
            return;
        }
        EnterChatRoomData enterChatRoomData = new EnterChatRoomData(roomId);
        String appKey = data != null ? (String) data.get("appKey") : null;
        String account = data != null ? (String) data.get("account") : null;
        String token = data != null ? (String) data.get("token") : null;
        String nick = data != null ? (String) data.get("nick") : null;
        String avatar = data != null ? (String) data.get("avatar") : null;
        Map extension = data != null ? (Map) data.get("extension") : null;
        Map notifyExtension = data != null ? (Map) data.get("notifyExtension") : null;
        enterChatRoomData.setAppKey(appKey);
        enterChatRoomData.setNick(nick);
        enterChatRoomData.setAvatar(avatar);
        enterChatRoomData.setExtension(extension);
        enterChatRoomData.setNotifyExtension(notifyExtension);
        final List<String> address = call.argument("links");
        int retryCount = call.argument("retryCount");
        if (address != null && address.size() > 0) {
            enterChatRoomData.setIndependentMode(new ChatRoomIndependentCallback() {

                @Override
                public List<String> getChatRoomLinkAddresses(String roomId, String account) {
                    return address;
                }
            }, account, token);
        }
        msgService.enterChatRoomEx(enterChatRoomData, retryCount).setCallback(
                new RequestCallbackWrapper<EnterChatRoomResultData>() {

                    @Override
                    public void onResult(int code, EnterChatRoomResultData result, Throwable exception) {
                        MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                        builder.put("code", code);
                        if (result != null) {
                            MapBuilder data = map();
                            data.put("roomId", result.getRoomId());
                            data.put("resCode", result.getResCode());
                            data.put("statusCode", StatusCodeUtil.convertStatusCode(result.getStatus()));
                            data.put("chatRoomInfo", buildChatRoomInfo(result.getRoomInfo()));
                            data.put("chatRoomMember", buildRoomMember(result.getMember()));
                            data.put("account", result.getAccount());
                            builder.put("enterDataResult", data.build());
                        }
                        channel.invokeMethod("onEnterChatRoom", builder.build());
                    }
                });
    }

    private Map<String, Object> buildRoomMember(ChatRoomMember info) {
        if (info == null) {
            return null;
        }
        MapBuilder builder = map();
        builder.put("roomId", info.getRoomId());
        builder.put("account", info.getAccount());
        builder.put("memberType", MemberTypeUtil.convertType(info.getMemberType()));
        builder.put("memberLevel", info.getMemberLevel());
        builder.put("nick", info.getNick());
        builder.put("avatar", info.getAvatar());
        builder.put("extension", info.getExtension());
        builder.put("online", info.isOnline());
        builder.put("inBlackList", info.isInBlackList());
        builder.put("isMuted", info.isMuted());
        builder.put("isValid", info.isValid());
        builder.put("enterTime", info.getEnterTime());
        builder.put("updateTime", info.getUpdateTime());
        builder.put("isTempMuted", info.isTempMuted());
        builder.put("tempMuteDuration", info.getTempMuteDuration());
        return builder.build();
    }

    private Map<String, Object> buildChatRoomInfo(ChatRoomInfo info) {
        if (info == null) {
            return null;
        }
        MapBuilder builder = map();
        builder.put("roomId", info.getRoomId());
        builder.put("name", info.getName());
        builder.put("announcement", info.getAnnouncement());
        builder.put("broadcastUrl", info.getBroadcastUrl());
        builder.put("creator", info.getCreator());
        builder.put("validFlag", info.isValid() ? 1 : 0);
        builder.put("onlineUserCount", info.getOnlineUserCount());
        builder.put("mute", info.isMute() ? 1 : 0);
        builder.put("extension", info.getExtension());
        builder.put("queueLevel", info.getQueueLevel());
        return builder.build();
    }
}
