/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

package im.netease.nim.nos;

import android.content.Context;
import android.text.TextUtils;

import com.netease.nimlib.sdk.NIMClient;
import com.netease.nimlib.sdk.Observer;
import com.netease.nimlib.sdk.RequestCallbackWrapper;
import com.netease.nimlib.sdk.ResponseCode;
import com.netease.nimlib.sdk.nos.NosService;
import com.netease.nimlib.sdk.nos.NosServiceObserve;
import com.netease.nimlib.sdk.nos.model.NosTransferInfo;
import com.netease.nimlib.sdk.nos.model.NosTransferProgress;

import java.io.File;

import im.netease.nim.base.Handler;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

/**
 * Created by hzsunyj on 2020-01-03.
 */
public class NosBridge extends Handler {

    public NosBridge(MethodChannel channel, Context context) {
        super(channel, context);
    }

    private NosService nosService;

    private NosServiceObserve nosServiceObserve;

    Observer<NosTransferInfo> observer = new Observer<NosTransferInfo>() {

        @Override
        public void onEvent(NosTransferInfo info) {
            MapBuilder builder = buildArg(observerModuleName());
            builder.put("transferType", info.getTransferType().ordinal());
            builder.put("path", info.getPath());
            builder.put("size", info.getSize());
            builder.put("md5", info.getMd5());
            builder.put("url", info.getUrl());
            builder.put("extension", info.getExtension());
            builder.put("status", info.getStatus().ordinal());
            channel.invokeMethod("observeNosTransferStatus", builder.build());
        }
    };

    Observer<NosTransferProgress> progressObserver = new Observer<NosTransferProgress>() {

        @Override
        public void onEvent(NosTransferProgress nosTransferProgress) {
            MapBuilder builder = buildArg(observerModuleName());
            builder.put("key", nosTransferProgress.getKey());
            builder.put("progress", (nosTransferProgress.getTransferred() * 1.0) / nosTransferProgress.getTotal());
            channel.invokeMethod("observeNosTransferProgress", builder.build());
        }
    };

    @Override
    public void unInit() {
        if (nosServiceObserve != null) {
            nosServiceObserve.observeNosTransferStatus(observer, false);
            nosServiceObserve.observeNosTransferProgress(progressObserver, false);
        }
        nosServiceObserve = null;
        nosService = null;
    }

    @Override
    public void postInit() {
        nosService = NIMClient.getService(NosService.class);
        nosServiceObserve = NIMClient.getService(NosServiceObserve.class);
    }

    @Override
    public void registerObserver() {
        nosServiceObserve.observeNosTransferStatus(observer, true);
        nosServiceObserve.observeNosTransferProgress(progressObserver, true);
    }

    @Override
    public String moduleName() {
        return "ResourceService";
    }
    @Override
    public String observerModuleName() {
        return "ResourceObserver";
    }
    @Override
    public void handle(String method, MethodCall call, MethodChannel.Result result) {
        if (TextUtils.isEmpty(method)) {
            return;
        }
        switch (method) {
            case "uploadAtScene":
                onUploadAtScene(call);
                break;
            case "getOriginUrlFromShortUrl":
                onGetOriginUrlFromShortUrl(call);
                break;
            case "convertDownloadUrlToCDNUrl":
                onConvertDownloadUrlToCDNUrl(call);
                break;
        }
    }

    private void onConvertDownloadUrlToCDNUrl(MethodCall call) {
        String url = call.argument("url");
        String newUrl = nosService.convertDownloadUrlToCDNUrl(url);
        MapBuilder builder = buildArg(getReceiptId(call), moduleName());
        builder.put("url", newUrl);
        channel.invokeMethod("onConvertDownloadUrlToCDNUrl", builder.build());
    }

    private void onGetOriginUrlFromShortUrl(final MethodCall call) {
        String roomId = call.argument("roomId");
        final String url = call.argument("shortUrl");
        nosService.getOriginUrlFromShortUrl(roomId, url).setCallback(new RequestCallbackWrapper<String>() {

            @Override
            public void onResult(int code, String s, Throwable throwable) {
                MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                builder.put("code", code);
                if (code == ResponseCode.RES_SUCCESS) {
                    builder.put("url", url);
                }
                channel.invokeMethod("onGetOriginUrlFromShortUrl", builder.build());
            }
        });
    }

    private void onUploadAtScene(final MethodCall call) {
        String filePath = call.argument("filePath");
        String mimeType = call.argument("mimeType");
        String sceneKey = call.argument("sceneKey");
        Boolean forceUpload1 = call.argument("forceUpload");
        boolean forceUpload = forceUpload1 == null ? false : forceUpload1;
        RequestCallbackWrapper callback = new RequestCallbackWrapper<String>() {

            @Override
            public void onResult(int code, String url, Throwable throwable) {
                MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                if (code == 200) {
                    builder.put("url", url);
                }
                channel.invokeMethod("onUploadAtScene", builder.build());
            }
        };
        if (TextUtils.isEmpty(sceneKey)) {
            nosService.upload(new File(filePath), mimeType).setCallback(callback);
        } else {
            nosService.uploadEnableForce(new File(filePath), mimeType, sceneKey, forceUpload).setCallback(callback);
        }
    }
}
