/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

package im.netease.nim.signalling;

import android.content.Context;
import android.text.TextUtils;

import com.netease.nimlib.sdk.NIMClient;
import com.netease.nimlib.sdk.Observer;
import com.netease.nimlib.sdk.RequestCallbackWrapper;
import com.netease.nimlib.sdk.avsignalling.SignallingService;
import com.netease.nimlib.sdk.avsignalling.SignallingServiceObserver;
import com.netease.nimlib.sdk.avsignalling.builder.CallParamBuilder;
import com.netease.nimlib.sdk.avsignalling.builder.InviteParamBuilder;
import com.netease.nimlib.sdk.avsignalling.constant.ChannelStatus;
import com.netease.nimlib.sdk.avsignalling.constant.ChannelType;
import com.netease.nimlib.sdk.avsignalling.event.ChannelCommonEvent;
import com.netease.nimlib.sdk.avsignalling.event.InviteAckEvent;
import com.netease.nimlib.sdk.avsignalling.event.InvitedEvent;
import com.netease.nimlib.sdk.avsignalling.event.MemberUpdateEvent;
import com.netease.nimlib.sdk.avsignalling.event.SyncChannelListEvent;
import com.netease.nimlib.sdk.avsignalling.event.UserJoinEvent;
import com.netease.nimlib.sdk.avsignalling.model.ChannelBaseInfo;
import com.netease.nimlib.sdk.avsignalling.model.ChannelFullInfo;
import com.netease.nimlib.sdk.avsignalling.model.MemberInfo;
import com.netease.nimlib.sdk.avsignalling.model.SignallingPushConfig;

import java.util.ArrayList;
import java.util.Map;

import im.netease.nim.base.Handler;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

/**
 * Created by hzsunyj on 2019-09-23.
 */
public class SignallingBridge extends Handler {

    public SignallingBridge(MethodChannel channel, Context context) {
        super(channel, context);
    }

    public SignallingService service;

    public SignallingServiceObserver observer;

    Observer<ChannelCommonEvent> onOnlineEvent = new Observer<ChannelCommonEvent>() {

        @Override
        public void onEvent(ChannelCommonEvent event) {
            MapBuilder builder = buildArg(observerModuleName());
            buildEvent(event, builder);
            channel.invokeMethod("onOnlineEvent", builder.build());
        }
    };

    Observer<ArrayList<ChannelCommonEvent>> offlineNotification = new Observer<ArrayList<ChannelCommonEvent>>() {

        @Override
        public void onEvent(ArrayList<ChannelCommonEvent> events) {
            MapBuilder builder = buildArg(observerModuleName());
            ArrayList<Map<String, Object>> arrayList = new ArrayList<>(events.size());
            for (ChannelCommonEvent event : events) {
                if (event == null) {
                    continue;
                }
                MapBuilder data = map();
                buildEvent(event, data);
                arrayList.add(data.build());
            }
            builder.put("events", arrayList);
            channel.invokeMethod("onOfflineEvent", builder.build());
        }
    };

    Observer<InviteAckEvent> inviteAckEventObserver = new Observer<InviteAckEvent>() {

        @Override
        public void onEvent(InviteAckEvent event) {
            MapBuilder builder = map();
            Map<String, Object> baseMap = buildBaseInfo(event.getChannelBaseInfo());
            if (baseMap != null) {
                builder.put("baseInfo", baseMap);
            }
            builder.put("fromAccountId", event.getFromAccountId());
            builder.put("customInfo", event.getCustomInfo());
            builder.put("type", event.getEventType().getValue());
            builder.put("time", event.getTime());
            builder.put("toAccountId", event.getToAccountId());
            builder.put("requestId", event.getRequestId());
            builder.put("status", event.getAckStatus().getValue());
            channel.invokeMethod("onOtherClientEvent", builder.build());
        }
    };

    Observer<ArrayList<SyncChannelListEvent>> syncChannelListNotification = new Observer<ArrayList<SyncChannelListEvent>>() {

        @Override
        public void onEvent(ArrayList<SyncChannelListEvent> events) {
            ArrayList<Map<String, Object>> arrayList = new ArrayList<>(events.size());
            for (SyncChannelListEvent event : events) {
                ChannelFullInfo fullInfo = event.getChannelFullInfo();
                if (fullInfo == null) {
                    continue;
                }
                MapBuilder builder = map(2);
                builder.put("baseInfo", buildBaseInfo(fullInfo.getChannelBaseInfo()));
                builder.put("members", convertMember(fullInfo.getMembers()));
                arrayList.add(builder.build());
            }
            MapBuilder builder = buildArg(observerModuleName());
            builder.put("events", arrayList);
            channel.invokeMethod("onSyncEvent", builder.build());
        }
    };

    Observer<MemberUpdateEvent> memberUpdateEventObserver = new Observer<MemberUpdateEvent>() {

        @Override
        public void onEvent(MemberUpdateEvent event) {
            ChannelFullInfo fullInfo = event.getChannelFullInfo();
            if (fullInfo == null) {
                return;
            }
            MapBuilder builder = buildArg(observerModuleName());
            ChannelBaseInfo baseInfo = fullInfo.getChannelBaseInfo();
            Map<String, Object> baseMap = buildBaseInfo(baseInfo);
            if (baseMap != null) {
                builder.put("baseInfo", baseMap);
            }
            builder.put("members", convertMember(fullInfo.getMembers()));
            channel.invokeMethod("onMemberUpdateEvent", builder.build());
        }
    };

    @Override
    public void unInit() {
        if (observer != null) {
            this.observer.observeOnlineNotification(onOnlineEvent, false);
            this.observer.observeOfflineNotification(offlineNotification, false);
            this.observer.observeOtherClientInviteAckNotification(inviteAckEventObserver, false);
            this.observer.observeSyncChannelListNotification(syncChannelListNotification, false);
            this.observer.observeMemberUpdateNotification(memberUpdateEventObserver, false);
        }
        observer = null;
        service = null;
    }

    @Override
    public void postInit() {
        service = NIMClient.getService(SignallingService.class);
        observer = NIMClient.getService(SignallingServiceObserver.class);
    }

    /**
     * may be unregister
     */
    @Override
    public void registerObserver() {
        this.observer.observeOnlineNotification(onOnlineEvent, true);
        this.observer.observeOfflineNotification(offlineNotification, true);
        this.observer.observeOtherClientInviteAckNotification(inviteAckEventObserver, true);
        this.observer.observeSyncChannelListNotification(syncChannelListNotification, true);
        this.observer.observeMemberUpdateNotification(memberUpdateEventObserver, true);
    }

    private void buildEvent(ChannelCommonEvent event, MapBuilder builder) {
        Map<String, Object> baseMap = buildBaseInfo(event.getChannelBaseInfo());
        if (baseMap != null) {
            builder.put("baseInfo", baseMap);
        }
        builder.put("fromAccountId", event.getFromAccountId());
        builder.put("customInfo", event.getCustomInfo());
        builder.put("type", event.getEventType().getValue());
        builder.put("time", event.getTime());
        switch (event.getEventType()) {
            case JOIN:
                UserJoinEvent joinEvent = (UserJoinEvent) event;
                builder.put("memberInfo", joinEvent.getMemberInfo().toString());
                break;
            case INVITE:
                InvitedEvent invitedEvent = (InvitedEvent) event;
                builder.put("toAccountId", invitedEvent.getToAccountId());
                builder.put("requestId", invitedEvent.getRequestId());
                SignallingPushConfig pushConfig = invitedEvent.getPushConfig();
                if (pushConfig != null) {
                    MapBuilder push = map();
                    push.put("needPush", pushConfig.needPush());
                    push.put("pushTitle", pushConfig.getPushTitle());
                    push.put("pushContent", pushConfig.getPushContent());
                    push.put("pushPayload", pushConfig.getPushPayload());
                    push.put("needBadge", false);
                    builder.put("pushConfig", push.build());
                }
                break;
            case REJECT:
            case ACCEPT:
                InviteAckEvent ackEvent = (InviteAckEvent) event;
                builder.put("toAccountId", ackEvent.getToAccountId());
                builder.put("requestId", ackEvent.getRequestId());
                builder.put("status", ackEvent.getAckStatus().getValue());
                break;

        }
    }

    private Map<String, Object> buildBaseInfo(ChannelBaseInfo channelBaseInfo) {
        if (channelBaseInfo != null) {
            MapBuilder baseInfo = map();
            baseInfo.put("channelName", channelBaseInfo.getChannelName());
            baseInfo.put("channelId", channelBaseInfo.getChannelId());
            baseInfo.put("type", channelBaseInfo.getType().getValue());
            baseInfo.put("channelExt", channelBaseInfo.getChannelExt());
            baseInfo.put("createTimestamp", channelBaseInfo.getCreateTimestamp());
            baseInfo.put("expireTimestamp", channelBaseInfo.getExpireTimestamp());
            baseInfo.put("creatorAccountId", channelBaseInfo.getCreatorAccountId());
            ChannelStatus status = channelBaseInfo.getChannelStatus();
            baseInfo.put("channelStatus", status != null ? status.getValue() : ChannelStatus.INVALID.getValue());
            return baseInfo.build();
        }
        return null;
    }

    @Override
    public String moduleName() {
        return "SignallingService";
    }
    @Override
    public String observerModuleName() {
        return "SignallingObserver";
    }

    @Override
    public void handle(String method, MethodCall call, MethodChannel.Result result) {
        if (TextUtils.isEmpty(method)) {
            return;
        }
        switch (method) {
            case "create":
                onCreate(call);
                break;
            case "close":
                onClose(call);
                break;
            case "join":
                onJoin(call);
                break;
            case "leave":
                onLeave(call);
                break;
            case "invite":
                onInvite(call);
                break;
            case "cancelInvite":
                onCancelInvite(call);
                break;
            case "rejectInvite":
                onRejectInvite(call);
                break;
            case "acceptInvite":
                onAcceptInvite(call);
                break;
            case "acceptInviteAndJoin":
                onAcceptInviteAndJoin(call);
                break;
            case "sendControl":
                onSendControl(call);
                break;
            case "call":
                onCall(call);
                break;
            case "queryChannelInfo":
                onQueryChannelInfo(call);
                break;
        }
        result.success(null);
    }

    /**
     * add new api query full， current only query base info
     *
     * @param call
     */
    private void onQueryChannelInfo(MethodCall call) {
        String channelName = call.argument("channelName");
        final String receiptId = call.argument("receiptId");
        service.queryChannelFullInfo(channelName).setCallback(new RequestCallbackWrapper<ChannelFullInfo>() {

            @Override
            public void onResult(int code, ChannelFullInfo result, Throwable exception) {
                callbackFullInfo("onQueryChannelInfo", code, result, receiptId);
            }
        });
    }

    private void onCall(final MethodCall call) {
        int type = call.argument("type");
        ChannelType type1 = ChannelType.retrieveType(type);
        String accountId = call.argument("accountId");
        String requestId = call.argument("requestId");
        String channelName = call.argument("channelName");
        String channelExt = call.argument("channelExt");
        Number uid = call.argument("selfUid");
        long selfUid = uid == null ? 0L : uid.longValue();
        Boolean enable = call.argument("offlineEnable");
        boolean offlineEnable = enable == null ? false : enable.booleanValue();
        String customInfo = call.argument("customInfo");
        Map pushConfig = call.argument("pushConfig");
        SignallingPushConfig config = null;
        if (pushConfig != null) {
            Boolean push = (Boolean) pushConfig.get("needPush");
            boolean needPush = push != null ? push.booleanValue() : false;
            String pushTitle = (String) pushConfig.get("pushTitle");
            String pushContent = (String) pushConfig.get("pushContent");
            String pushPayload = (String) pushConfig.get("pushPayload");
            Boolean badge = (Boolean) pushConfig.get("needBadge");
            boolean needBadge = badge != null ? badge.booleanValue() : false;
            config = new SignallingPushConfig(needPush, pushTitle, pushContent);
        }
        CallParamBuilder builder = new CallParamBuilder(type1, accountId, requestId);
        builder.channelName(channelName);
        builder.channelExt(channelExt);
        builder.selfUid(selfUid);
        builder.customInfo(customInfo);
        builder.offlineEnabled(offlineEnable);
        builder.pushConfig(config);
        service.call(builder).setCallback(new RequestCallbackWrapper<ChannelFullInfo>() {

            @Override
            public void onResult(int code, ChannelFullInfo result, Throwable exception) {
                callbackFullInfo("onAcceptInviteAndJoin", code, result, getReceiptId(call));
            }
        });
    }

    private void onSendControl(final MethodCall call) {
        String channelId = call.argument("channelId");
        String accountId = call.argument("accountId");
        String customInfo = call.argument("customInfo");
        service.sendControl(channelId, accountId, customInfo).setCallback(new RequestCallbackWrapper<Void>() {

            @Override
            public void onResult(int code, Void result, Throwable exception) {
                callbackCode("onSendControl", code, getReceiptId(call));
            }
        });
    }

    private void onAcceptInviteAndJoin(final MethodCall call) {
        String channelId = call.argument("channelId");
        String accountId = call.argument("accountId");
        String requestId = call.argument("requestId");
        String customInfo = call.argument("customInfo");
        Boolean enable = call.argument("offlineEnable");
        boolean offlineEnable = enable == null ? false : enable.booleanValue();
        Number uid = call.argument("selfUid");
        long selfUid = uid == null ? 0L : uid.longValue();
        InviteParamBuilder builder = new InviteParamBuilder(channelId, accountId, requestId);
        builder.customInfo(customInfo);
        builder.offlineEnabled(offlineEnable);
        service.acceptInviteAndJoin(builder, selfUid).setCallback(new RequestCallbackWrapper<ChannelFullInfo>() {

            @Override
            public void onResult(int code, ChannelFullInfo result, Throwable exception) {
                callbackFullInfo("onAcceptInviteAndJoin", code, result, getReceiptId(call));
            }
        });
    }

    private void onAcceptInvite(final MethodCall call) {
        String channelId = call.argument("channelId");
        String accountId = call.argument("accountId");
        String requestId = call.argument("requestId");
        String customInfo = call.argument("customInfo");
        Boolean enable = call.argument("offlineEnable");
        boolean offlineEnable = enable == null ? false : enable.booleanValue();
        InviteParamBuilder builder = new InviteParamBuilder(channelId, accountId, requestId);
        builder.customInfo(customInfo);
        builder.offlineEnabled(offlineEnable);
        service.acceptInvite(builder).setCallback(new RequestCallbackWrapper<Void>() {

            @Override
            public void onResult(int code, Void result, Throwable exception) {
                callbackCode("onRejectInvite", code, getReceiptId(call));
            }
        });
    }

    private void onRejectInvite(final MethodCall call) {
        String channelId = call.argument("channelId");
        String accountId = call.argument("accountId");
        String requestId = call.argument("requestId");
        String customInfo = call.argument("customInfo");
        Boolean enable = call.argument("offlineEnable");
        boolean offlineEnable = enable == null ? false : enable.booleanValue();
        InviteParamBuilder builder = new InviteParamBuilder(channelId, accountId, requestId);
        builder.customInfo(customInfo);
        builder.offlineEnabled(offlineEnable);
        service.rejectInvite(builder).setCallback(new RequestCallbackWrapper<Void>() {

            @Override
            public void onResult(int code, Void result, Throwable exception) {
                callbackCode("onRejectInvite", code, getReceiptId(call));
            }
        });
    }

    private void onCancelInvite(final MethodCall call) {
        String channelId = call.argument("channelId");
        String accountId = call.argument("accountId");
        String requestId = call.argument("requestId");
        String customInfo = call.argument("customInfo");
        Boolean enable = call.argument("offlineEnable");
        boolean offlineEnable = enable == null ? false : enable.booleanValue();
        InviteParamBuilder builder = new InviteParamBuilder(channelId, accountId, requestId);
        builder.customInfo(customInfo);
        builder.offlineEnabled(offlineEnable);
        service.cancelInvite(builder).setCallback(new RequestCallbackWrapper<Void>() {

            @Override
            public void onResult(int code, Void result, Throwable exception) {
                callbackCode("onCancelInvite", code, getReceiptId(call));
            }
        });
    }

    private void onInvite(final MethodCall call) {
        String channelId = call.argument("channelId");
        String accountId = call.argument("accountId");
        String requestId = call.argument("requestId");
        String customInfo = call.argument("customInfo");
        Map pushConfig = call.argument("pushConfig");
        SignallingPushConfig config = null;
        if (pushConfig != null) {
            boolean needPush = (boolean) pushConfig.get("needPush");
            String pushTitle = (String) pushConfig.get("pushTitle");
            String pushContent = (String) pushConfig.get("pushContent");
            String pushPayload = (String) pushConfig.get("pushPayload");
            boolean needBadge = (boolean) pushConfig.get("needBadge");
            config = new SignallingPushConfig(needPush, pushTitle, pushContent);
        }
        Boolean enable = call.argument("offlineEnable");
        boolean offlineEnable = enable == null ? false : enable.booleanValue();
        InviteParamBuilder builder = new InviteParamBuilder(channelId, accountId, requestId);
        builder.customInfo(customInfo);
        builder.pushConfig(config);
        builder.offlineEnabled(offlineEnable);
        service.invite(builder).setCallback(new RequestCallbackWrapper<Void>() {

            @Override
            public void onResult(int code, Void result, Throwable exception) {
                callbackCode("onInvite", code, getReceiptId(call));
            }
        });
    }

    private void onLeave(final MethodCall call) {
        String channelId = call.argument("channelId");
        Boolean enable = call.argument("offlineEnable");
        boolean offlineEnable = enable == null ? false : enable.booleanValue();
        String customInfo = call.argument("customInfo");
        service.leave(channelId, offlineEnable, customInfo).setCallback(new RequestCallbackWrapper<Void>() {

            @Override
            public void onResult(int code, Void result, Throwable exception) {
                callbackCode("onLeave", code, getReceiptId(call));
            }
        });
    }
    private void callbackCode(String methodName, int code, String receiptId) {
        MapBuilder builder = buildArg(receiptId, moduleName());
        builder.put("code", code);
        Map<String, Object> arguments = builder.build();
        channel.invokeMethod(methodName, arguments);
    }

    private void onJoin(final MethodCall call) {
        String channelId = call.argument("channelId");
        Number uid = call.argument("selfUid");
        long selfUid = uid == null ? 0L : uid.longValue();
        String customInfo = call.argument("customInfo");
        Boolean enable = call.argument("offlineEnable");
        boolean offlineEnable = enable == null ? false : enable.booleanValue();
        service.join(channelId, selfUid, customInfo, offlineEnable).setCallback(
                new RequestCallbackWrapper<ChannelFullInfo>() {

                    @Override
                    public void onResult(int code, ChannelFullInfo result, Throwable exception) {
                        callbackFullInfo("onJoin", code, result, getReceiptId(call));
                    }
                });
    }

    private void callbackFullInfo(String methodName, int code, ChannelFullInfo result, String receiptId) {
        MapBuilder builder = buildArg(receiptId, moduleName());
        builder.put("code", code);
        if (code == 200) {
            MapBuilder baseInfo = map();
            baseInfo.put("channelName", result.getChannelName());
            baseInfo.put("channelId", result.getChannelId());
            baseInfo.put("type", result.getType().getValue());
            baseInfo.put("channelExt", result.getChannelExt());
            baseInfo.put("createTimestamp", result.getCreateTimestamp());
            baseInfo.put("expireTimestamp", result.getExpireTimestamp());
            baseInfo.put("creatorAccountId", result.getCreatorAccountId());
            baseInfo.put("channelStatus", result.getChannelStatus());
            MapBuilder fullInfo = map();
            fullInfo.put("channelBaseInfo", baseInfo.build());
            fullInfo.put("members", convertMember(result.getMembers()));
            builder.put("channelFullInfo", fullInfo.build());
        }
        Map<String, Object> arguments = builder.build();
        channel.invokeMethod(methodName, arguments);
    }

    private ArrayList<String> convertMember(ArrayList<MemberInfo> members) {
        if (members == null || members.size() == 0) {
            return null;
        }
        ArrayList<String> arrayList = new ArrayList<>(members.size());
        for (MemberInfo memberInfo : members) {
            arrayList.add(memberInfo.toString());
        }
        return arrayList;
    }

    private void onClose(final MethodCall call) {
        String channelId = call.argument("channelId");
        Boolean enable = call.argument("offlineEnable");
        boolean offlineEnable = enable == null ? false : enable.booleanValue();
        String customInfo = call.argument("customInfo");
        service.close(channelId, offlineEnable, customInfo).setCallback(new RequestCallbackWrapper<Void>() {

            @Override
            public void onResult(int code, Void result, Throwable exception) {
                callbackCode("onCreate", code, getReceiptId(call));
            }
        });
    }

    private void onCreate(final MethodCall call) {
        Number inType = call.argument("type");
        int type = inType != null ? inType.intValue() : ChannelType.CUSTOM.getValue();
        ChannelType type1 = ChannelType.retrieveType(type);
        String channelName = call.argument("channelName");
        String channelExt = call.argument("channelExt");
        service.create(type1, channelName, channelExt).setCallback(new RequestCallbackWrapper<ChannelBaseInfo>() {

            @Override
            public void onResult(int code, ChannelBaseInfo result, Throwable exception) {
                MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                builder.put("code", code);
                if (result != null) {
                    MapBuilder data = map();
                    data.put("channelName", result.getChannelName());
                    data.put("channelId", result.getChannelId());
                    data.put("type", result.getType().getValue());
                    data.put("channelExt", result.getChannelExt());
                    data.put("createTimestamp", result.getCreateTimestamp());
                    data.put("expireTimestamp", result.getExpireTimestamp());
                    data.put("creatorAccountId", result.getCreatorAccountId());
                    data.put("channelStatus", result.getChannelStatus());
                    builder.put("channelBaseInfo", data.build());
                }
                channel.invokeMethod("onCreate", builder.build());
            }
        });
    }
}
