// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

/// 进入聊天室数据
class EnterChatRoomData {
  /// 必填,聊天室ID
  String? roomId;

  /// 聊天室可以使用独立的appkey， 不依赖于im appkey， 如果不传，则使用im appkey
  String? appKey;

  /// 登录用户名，不填则表示匿名
  String? account;

  /// 登录token
  String? token;

  /// 进入聊天室后展示的昵称
  String? nick;

  /// 进入聊天室后展示的头像
  String? avatar;

  /// 进入聊天室后展示的扩展字段，长度限制4K
  Map<String, dynamic>? extension;

  /// 进入聊天室通知开发者扩展字段，长度限制2K
  Map<String, dynamic>? notifyExtension;

  Map<String, dynamic> _toMap() => {
        "roomId": roomId,
        "appKey": appKey,
        "account": account,
        "token": token,
        "nick": nick,
        "avatar": avatar,
        "extension": extension,
        "notifyExtension": notifyExtension
      };
}

/// 进入聊天室回包数据
class EnterChatRoomResultData {
  /// 聊天室id
  String? roomId;

  /// 错误码
  int? resCode;

  /// 状态码
  StatusCode? statusCode;

  /// 聊天室信息
  ChatRoomInfo? chatRoomInfo;

  /// 聊天室成员信息
  ChatRoomMember? chatRoomMember;

  /// 账号， 比如匿名，是native生成的
  String? account;

  ///crate EnterChatRoomResultData object from map
  static EnterChatRoomResultData _fromMap(arg) {
    EnterChatRoomResultData data = EnterChatRoomResultData();
    if(arg != null) {
      data.roomId = arg['roomId'];
      data.resCode = arg['resCode'];
      int value = arg['statusCode'];
      StatusCode statusCode =
      value >= 0 && value < StatusCode.values.length
          ? StatusCode.values[value]
          : StatusCode.invalid;
      data.statusCode = statusCode;
      data.chatRoomInfo = ChatRoomInfo._fromMap(arg['chatRoomInfo']);
      data.chatRoomMember = ChatRoomMember._fromMap(arg['chatRoomMember']);
      data.account = arg['account'];
    }
    return data;
  }

  Map<String, dynamic> _toMap() =>{
    "roomId":roomId,
    "statusCode":statusCode,
    "chatRoomInfo":chatRoomInfo?._toMap(),
    "chatRoomMember":chatRoomMember?._toMap(),
    "account":account,
  };
}

/// 聊天室信息
class ChatRoomInfo {
  /// 聊天室id
  String? roomId;

  /// 聊天室名称
  String? name;

  /// 聊天室公告
  String? announcement;

  /// 视频直播拉流地址
  String? broadcastUrl;

  /// 聊天室创建者账号
  String? creator;

  /// 聊天室有效标记, 1:有效,0:无效
  int? validFlag;

  /// 当前在线用户数量
  int? onlineUserCount;

  /// 聊天室禁言标记
  int? mute;

  /// 第三方扩展字段, 长度4k
  Map<String, dynamic>? extension;

  /// 队列管理权限，如是否有权限提交他人key和信息到队列中
  int? queueLevel;

  static ChatRoomInfo? _fromMap(arg) {
    if (arg == null) {
      return null;
    }
    ChatRoomInfo roomInfo = ChatRoomInfo();
    roomInfo.roomId = arg['roomId'];
    roomInfo.name = arg['name'];
    roomInfo.announcement = arg['announcement'];
    roomInfo.broadcastUrl = arg['broadcastUrl'];
    roomInfo.creator = arg['creator'];
    roomInfo.validFlag = arg['validFlag'];
    roomInfo.onlineUserCount = arg['onlineUserCount'];
    roomInfo.mute = arg['mute'];
    roomInfo.extension = arg['extension'];
    roomInfo.queueLevel = arg['queueLevel'];
    return roomInfo;
  }

  Map<String, dynamic> _toMap() => {
    "roomId":roomId,
    "name":name,
    "announcement":announcement,
    "broadcastUrl":broadcastUrl,
    "creator":creator,
    "validFlag":validFlag,
    "onlineUserCount":onlineUserCount,
    "mute":mute,
    "extension":extension,
    "queueLevel":queueLevel,
  };
}

/// 聊天室成员信息
class ChatRoomMember {
  /// 聊天室id
  String? roomId;

  ///成员账号
  String? account;

  ///成员类型：主要分为游客和非游客，非游客又分成受限用户、普通用户、创建者、管理员;
  late MemberType memberType;

  ///成员级别：大于等于0表示用户开发者可以自定义的级别
  int? memberLevel;

  /*** 以下字段进入聊天室提交****************/

  /// 聊天室内的昵称字段，预留字段，可从NimUserInfo中取，也可以由用户进聊天室时提交。
  String? nick;

  ///聊天室内的头像，预留字段，可从NimUserInfo中取avatar，可以由用户进聊天室时提交
  String? avatar;

  /// 开发者扩展字段，长度限制4k，可以由用户进聊天室时提交
  Map<String, dynamic>? extension;

  /*** 以下字段进入聊天室提交****************/

  /// 成员是否处于在线状态，仅特殊成员才可能离线，对游客/匿名用户而言只能是在线
  bool? online;

  ///是否在黑名单中
  bool? inBlackList;

  ///是禁言用户
  bool? isMuted;

  ///记录有效标记位
  bool? isValid;

  ///进入聊天室的时间点,对于离线成员该字段为空
  int? enterTime;

  ///固定成员的记录更新时间,用于固定成员列表的排列查询
  int? updateTime;

  ///是否临时禁言
  bool? isTempMuted;

  /// 临时禁言的解除时长,单位秒
  int? tempMuteDuration;

  static ChatRoomMember? _fromMap(arg) {
    if (arg == null) {
      return null;
    }
    ChatRoomMember member = ChatRoomMember();
    member.roomId = arg['roomId'];
    member.account = arg['account'];
    member.memberType = MemberType.values[arg['memberType']];
    member.memberLevel = arg['memberLevel'];
    member.nick = arg['nick'];
    member.avatar = arg['avatar'];
    member.extension = arg['extension'];
    member.online = arg['online'];
    member.inBlackList = arg['inBlackList'];
    member.isMuted = arg['isMuted'];
    member.isValid = arg['isValid'];
    member.enterTime = arg['enterTime'];
    member.updateTime = arg['updateTime'];
    member.isTempMuted = arg['isTempMuted'];
    member.tempMuteDuration = arg['tempMuteDuration'];
    return member;
  }

  Map<String, dynamic> _toMap() => {
        'roomId': roomId,
        'account': account,
        'memberType': memberType.index,
        'memberLevel': memberLevel,
        'nick': nick,
        'avatar': avatar,
        'extension': extension,
        'online': online,
        'inBlackList': inBlackList,
        'isMuted': isMuted,
        'isValid': isValid,
        'enterTime': enterTime,
        'updateTime': updateTime,
        'isTempMuted': isTempMuted,
        'tempMuteDuration': tempMuteDuration,
      };
}

/// platform define different， need convert
enum MemberType {
  /// 未知
  unknown,

  /// 游客
  guest,

  /// 受限用户
  limited,

  /// 普通成员（非游客）
  normal,

  /// 创建者（非游客）
  creator,

  /// 管理员（非游客）
  admin,

  /// 匿名游客
  anonymous,
}

/// 聊天室消息
class ChatRoomMessage extends IMMessage {
  /// 获取聊天室消息扩展属性
  ///
  /// @return 聊天室消息扩展属性
  ChatRoomMessageExtension? messageExtension;

  /// 获取聊天室消息配置
  ///
  /// @return 聊天室消息配置
  CustomChatRoomMessageConfig? roomMessageConfig;

  /// 是否是高优先级消息
  bool? isHighPriorityMessage;

  Map<String, dynamic> _toMap() => {
        'messageExtension': messageExtension?._toMap(),
        'roomMessageConfig': roomMessageConfig?._toMap(),
        ...super._toMap()
      };

  static _fromMap(map) {
    ChatRoomMessage message = ChatRoomMessage();
    IMMessage._parserMap(message, map);
    message.messageExtension = ChatRoomMessageExtension._fromMap(map['messageExtension']);
    message.roomMessageConfig = CustomChatRoomMessageConfig._fromMap(map['roomMessageConfig']);
    return message;
  }
}

/// 聊天室消息扩展字段
class ChatRoomMessageExtension {
  int roleInfoTimeTag = -1;

  String? nick;

  String? avatar;

  Map<String, dynamic>? senderExtension;

  Map<String, dynamic> _toMap() =>
      {'roleInfoTimeTag': roleInfoTimeTag, 'nick': nick, 'avatar': avatar, 'senderExtension': senderExtension};

  static ChatRoomMessageExtension? _fromMap(var map) {
    if (map == null) {
      return null;
    }
    ChatRoomMessageExtension extension = ChatRoomMessageExtension();
    extension.nick = map['nick'];
    extension.avatar = map['avatar'];
    extension.senderExtension = map['senderExtension'];
    return extension;
  }
}

/// 自定义聊天室消息配置
class CustomChatRoomMessageConfig {
  /// 该消息是否要保存到服务器，如果为true，通过{@link ChatRoomService#pullMessageHistory(String, long, int)}拉取的结果将不包含该条消息。
  /// 默认为false。
  bool? skipHistory = false;

  Map<String, dynamic> _toMap() => {'skipHistory': skipHistory};

  static CustomChatRoomMessageConfig? _fromMap(map) {
    if (map == null) {
      return null;
    }
    CustomChatRoomMessageConfig config = CustomChatRoomMessageConfig();
    config.skipHistory = map['skipHistory'];
    return config;
  }
}

/// 成员查询类型
enum MemberQueryType {
  /// 未知
  unknown,

  /// 固定成员（包括创建者,管理员,普通等级用户,受限用户(禁言+黑名单),即使非在线也可以在列表中看到,有数量限制 ）
  normal,

  ///非固定成员 (又称临时成员,只有在线时才能在列表中看到,数量无上限)
  guest,

  ///仅在线的固定成员
  onlineNormal
}

/// 成员变更数据字段
class ChatRoomMemberUpdate{

  /// 聊天室内昵称
  String? nick;

  /// 聊天室内头像
  String? avatar;

  /// 聊天室内扩展字段
  Map<String, dynamic>? extension;

  /// 更新的信息是否持久化，只对固定成员生效
  bool? needSave;

  Map<String, dynamic> _toMap() => {'nick': nick, 'avatar': avatar, 'extension': extension, 'needSave': needSave};
}
