// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

typedef RoomLinkAddress = Future<List<String>?> Function(String? roomId, String? account);

class ChatRoomService extends Service {
  ChatRoomService(MethodChannel methodChannel, Map<String, Handler> handlerMap) : super(methodChannel, handlerMap);

  @override
  String _getModule() {
    return "ChatRoomService";
  }

  /// 进入聊天室
  /// @param data 聊天室基本数据（聊天室ID必填）
  /// @param retryCount 重试次数
  Future<void> enterChatRoom(EnterChatRoomData data, int retryCount, Callback<EnterChatRoomResultData> callback,
      {RoomLinkAddress? linkAddress}) async {
    List<String>? links;
    if (linkAddress != null) {
      links = await linkAddress(data.roomId, data.account);
      if (links == null || links.length <= 0) {
        callback(NimErrorCode.callbackException, null);
      }
    }
    Map map = buildArguments(callback: callback);
    map['data'] = data._toMap();
    map['retryCount'] = retryCount;
    map['links'] = links;
    return await _methodChannel.invokeMethod('enterChatRoom', map);
  }

  ///
  /// 获取进入聊天室失败的错误码
  /// 如果是手动登录，在enterChatRoom的回调函数中已有错误码。
  /// 如果是断网重连，在自动登录失败时，即监听到在线状态变更为UNLOGIN时，可以采用此接口查看具体自动登录失败的原因。
  ///
  /// @param roomId 聊天室ID
  /// @return 聊天室断网重连、自动登录失败的错误码
  ///
  Future<int?> getEnterErrorCode(String roomId) async {
    Map map = buildArguments();
    map['roomId'] = roomId;
    return await _methodChannel.invokeMethod('getEnterErrorCode', map);
  }

  /// 离开聊天室
  ///
  /// @param roomId 聊天室ID
  Future<void> exitChatRoom(String roomId) async {
    Map map = buildArguments();
    map['roomId'] = roomId;
    return await _methodChannel.invokeMethod('exitChatRoom', map);
  }

  /// 发送消息
  ///
  /// @param msg    带发送的消息体，由{@link ChatRoomMessageBuilder}构造。
  /// @param resend 如果是发送失败后重发，标记为true，否则填false。
  /// @return callback 可以设置回调函数。消息发送完成后才会调用，如果出错，会有具体的错误代码。
  Future<void> sendMessage(ChatRoomMessage message, bool resend, Callback<void> callback) async {
    Map map = buildArguments(callback: callback);
    map['message'] = message._toMap();
    map['resend'] = resend;
    return await _methodChannel.invokeMethod('sendMessage', map);
  }

  /// 正常情况收到消息后附件会自动下载。如果下载失败，可调用该接口重新下载。
  ///
  /// @param msg   附件所在的消息体
  /// @param thumb 下载缩略图还是原文件。为true时，仅下载缩略图。<br>
  ///              该参数仅对图片和视频类消息有效
  /// @return callback 调用跟踪。可设置回调函数，可中止下载操作
  Future<void> downloadAttachment(ChatRoomMessage message, bool thumb, Callback<void> callback) async {
    Map map = buildArguments(callback: callback);
    map['message'] = message._toMap();
    map['thumb'] = thumb;
    return await _methodChannel.invokeMethod('downloadAttachment', map);
  }

  /// 获取历史消息,可选择给定时间往前或者往后查询，以及查询指定一种或多种类型的消息。
  /// <p>
  /// 若方向往前，则结果排序按时间逆序，反之则结果排序按时间顺序
  /// <p>
  /// 消息类型仅支持 0:文本，1:图片，2:语音，3:视频，4:地理位置，5:通知，6:文件，10:提示，11:Robot，100:自定义，其它为非法参数
  ///
  /// @param roomId    聊天室id
  /// @param startTime 时间戳，单位毫秒
  /// @param limit     可拉取的消息数量, default 20
  /// @param direction 查询方向
  /// @param typeEnums 消息类型，数组
  /// @return callback 可以设置回调函数。回调中返回历史消息列表
  Future<void> pullMessageHistory(String roomId, int startTime, int limit, Callback<List<ChatRoomMessage>> callback,
      {bool queryNew = false, List<MsgType>? types}) async {
    Map map = buildArguments(callback: callback);
    map['roomId'] = roomId;
    map['startTime'] = startTime;
    map['limit'] = limit;
    map['queryNew'] = queryNew;
    map['types'] = types;
    return await _methodChannel.invokeMethod('pullMessageHistory', map);
  }

  /// 获取当前聊天室信息
  ///
  /// @param roomId 聊天室id
  /// @return callback 可以设置回调函数。回调中返回聊天室的信息
  Future<void> fetchRoomInfo(String roomId, Callback<ChatRoomInfo> callback) async {
    Map map = buildArguments(callback: callback);
    map['roomId'] = roomId;
    return await _methodChannel.invokeMethod('fetchRoomInfo', map);
  }

  /// 获取聊天室成员信息
  ///
  /// @param roomId          聊天室id
  /// @param memberQueryType 成员查询类型。见{@link MemberQueryType}
  /// @param time            固定成员列表用updateTime,
  ///                        游客列表用进入enterTime，
  ///                        填0会使用当前服务器最新时间开始查询，即第一页，单位毫秒
  /// @param limit           条数限制
  /// @return callback 可以设置回调函数。回调中返回成员信息列表
  Future<void> fetchRoomMembers(String roomId, MemberQueryType type, ChatRoomMember member, int limit,
      Callback<List<ChatRoomMember>> callback) async {
    Map map = buildArguments(callback: callback);
    map['roomId'] = roomId;
    map['type'] = type.index;
    map['member'] = member._toMap();
    map['limit'] = limit;
    return await _methodChannel.invokeMethod('fetchRoomMembers', map);
  }

  ///
  /// @param roomId          聊天室id
  /// @param update          成员变更信息
  /// @param needNotify      是否需要通知
  /// @param notifyExtension 通知的扩展内容, dynamic需要可序列化
  ///
  Future<void> updateMyRoomRole(String roomId, ChatRoomMemberUpdate update, bool needNotify, Map<String, dynamic> notifyExtension, Callback<void> callback) async {
    Map map = buildArguments(callback: callback);
    map['roomId'] = roomId;
    map['memberUpdate'] = update._toMap();
    map['needNotify'] = needNotify;
    map['notifyExtension'] = notifyExtension;
    return await _methodChannel.invokeMethod('updateMyRoomRole', map);
  }

  @override
  Future? _handlerMethod(String method, int code, Map<dynamic, dynamic> arg, callback) {
    switch (method) {
      case "onEnterChatRoom":
        EnterChatRoomResultData data = EnterChatRoomResultData._fromMap(arg['enterDataResult']);
        callback(code, data);
        break;
      case 'onSendMessage':
        callback(code, null);
        break;
      case 'onDownloadAttachment':
        callback(code, null);
        break;
      case 'onFetchRoomInfo':
        ChatRoomInfo? chatRoomInfo = ChatRoomInfo._fromMap(arg['roomInfo']);
        callback(code, chatRoomInfo);
        break;
      case 'onPullMessageHistory':
        List? messages = arg['messages'];
        List<ChatRoomMessage>? list =
            messages != null ? messages.map<IMMessage>((i) => ChatRoomMessage._fromMap(i)).toList() as List<ChatRoomMessage>? : null;
        callback(code, list);
        break;
      case 'onFetchRoomMembers':
        List? messages = arg['members'];
        List<ChatRoomMember?>? list =
            messages != null ? messages.map<ChatRoomMember?>((i) => ChatRoomMember._fromMap(i)).toList() : null;
        callback(code, list);
        break;
      case 'onUpdateMyRoomRole':
        callback(code, null);
        break;
    }
    return null;
  }
}
