// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

class UserObserver extends Observable {
  static List<Observer<User?>> _friendChangedObservers = [];
  static List<Observer<User>> _blackedListChangedObservers = [];
  static List<Observer<User>> _mutedListChangedObservers = [];
  static List<Observer<User?>> _userInfoChangedListChangedObservers = [];

  UserObserver(MethodChannel methodChannel, Map<String, Handler> handlerMap) : super(methodChannel, handlerMap);

  @override
  Future<dynamic>? _handlerMethod(String method, Map? map) {
    switch (method) {
      case 'onFriendChanged':
        _handleFriendChanged(map!);
        break;
      case 'onBlackListChanged':
        _handleBlackListChanged();
        break;
      case 'onMuteListChanged':
        _handleMutedListChanged();
        break;
      case 'onUserInfoChanged':
        _handleUserInfoChanged(map!);
        break;
    }
    return null;
  }

  @override
  String _getModule() => 'UserObserver';

  /// 监听好友状态变化
  void registerFriendChange(Observer<User> observer) {
    _registerObserver(observer, _friendChangedObservers);
  }

  /// 注销监听好友状态变化
  void unRegisterFriendChange(Observer<User> observer) {
    _unregisterObserver(observer, _friendChangedObservers);
  }

  /// 监听黑名单列表变化
  void registerBlackedListChange(Observer<void> observer) {
    _registerObserver(observer, _blackedListChangedObservers);
  }

  /// 注销监听黑名单列表变化
  void unRegisterBlackedListChange(Observer<void> observer) {
    _unregisterObserver(observer, _blackedListChangedObservers);
  }

  /// 注册监听禁言列表变化
  void registerMutedListChange(Observer<void> observer) {
    _registerObserver(observer, _mutedListChangedObservers);
  }

  /// 注销监听禁言列表变化
  void unRegisterMutedListChange(Observer<void> observer) {
    _unregisterObserver(observer, _mutedListChangedObservers);
  }

  /// 监听用户资料变化
  void registerUserInfoChange(Observer<User> observer) {
    _registerObserver(observer, _userInfoChangedListChangedObservers);
  }

  /// 注销监听用户资料变化
  void unRegisterUserInfoChange(Observer<User> observer) {
    _unregisterObserver(observer, _userInfoChangedListChangedObservers);
  }

  // 处理好友关系变化

  void _handleFriendChanged(Map map) {
    User? changedFriend = map['friend'];
    for (Observer<User?> observer in _friendChangedObservers) {
      observer(changedFriend);
    }
  }

  // 黑名单列表变化
  void _handleBlackListChanged() {
    for (Observer<void> observer in _blackedListChangedObservers as Iterable<void Function(void)>) {
      observer(null);
    }
  }

  // 禁言列表变化
  void _handleMutedListChanged() {
    for (Observer<void> observer in _mutedListChangedObservers as Iterable<void Function(void)>) {
      observer(null);
    }
  }

  // 处理用户资料变化

  void _handleUserInfoChanged(Map map) {
    User? changedFriend = map['friend'];
    for (Observer<User?> observer in _userInfoChangedListChangedObservers) {
      observer(changedFriend);
    }
  }
}
