// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

class UserService extends Service {
  UserService(MethodChannel _methodChannel, Map<String, Handler> handlerMap) : super(_methodChannel, handlerMap);

  @override
  String _getModule() {
    return "UserService";
  }

  @override
  Future<dynamic>? _handlerMethod(String method, int code, Map arg, dynamic callback) {
    switch (method) {
      case 'onRequestFriend':
      case 'onDeleteFriend':
      case 'onAddToBlackList':
      case 'onRemoveFromBlackList':
      case 'onUpdateNotifyState':
      case 'onUpdateFriend':
        callback(arg['code'], null);
        break;
      case 'onFetchUsers':
        _handleFetchUserInfos(arg,callback);
       break;
    }
    return null;
  }

  ///
  /// 请求好友
  ///
  /// @param request 请求参数
  /// @param callback 回调
  ///
  Future<void> requestFriend(FriendRequest request, Callback callback) async {
    Map args = buildArguments(arg: request._toMap(), callback: callback);
    return await _methodChannel.invokeMethod('requestFriend', args);
  }

  ///
  /// 删除好友
  ///
  /// @param userId 用户ID
  /// @param callback 完成回调
  /// @param removeAlias 是否同时删除备注名，可选填，默认不删除
  ///
  Future<void> deleteFriend(String userId, Callback callback, [bool removeAlias = false]) async {
    Map args = buildArguments(arg: {'userId': userId, 'removeAlias': removeAlias}, callback: callback);
    return await _methodChannel.invokeMethod('deleteFriend', args);
  }

  ///
  /// 返回我的好友列表
  ///
  /// @return 好友列表
  ///
  Future<List<User>?> myFriends() async {
    List? list = await _methodChannel.invokeMethod('myFriends');
    return _friendListFromList(list);
  }

  ///
  /// 判断是否是我的好友
  ///
  /// @param userId 用户ID
  /// @return 是否是我的好友
  ///
  Future<bool?> isMyFriend(String userId) async {
    Map args = buildArguments(arg: {'userId': userId});
    return await _methodChannel.invokeMethod('isMyFriend', args);
  }

  ///
  /// 添加到黑名单
  ///
  /// @param userId 用户ID
  ///
  Future<void> addToBlackList(String userId) async {
    Map args = buildArguments(arg: {'userId': userId});
    return await _methodChannel.invokeMethod('addToBlackList', args);
  }

  ///
  /// 将用户移除黑名单
  ///
  /// @param userId 用户Id
  ///
  Future<void> removeFromBlackList(String userId, Callback callback) async {
    Map args = buildArguments(arg: {'userId': userId}, callback: callback);
    return await _methodChannel.invokeMethod('removeFromBlackList', args);
  }

  ///
  /// 判断用户是否已被拉黑
  ///
  /// @param userId 用户ID
  /// @return 是否在黑名单
  ///
  Future<bool?> isInBlackList(String userId) async {
    Map args = buildArguments(arg: {'userId': userId});
    return await _methodChannel.invokeMethod('isInBlackList', args);
  }

  ///
  /// 返回所有在黑名单中的用户列表
  ///
  /// @return 黑名单列表
  ///
  Future<List<User>?> myBlackList() async {
    List? list = await _methodChannel.invokeMethod('myBlackList');
    return _friendListFromList(list);
  }

  ///
  /// 设置消息提醒
  ///
  /// @param userId       用户Id
  /// @param notify       是否提醒
  /// @param callback     完成回调
  ///
  Future<void> updateNotifyState(String userId, bool notify, Callback callback) async {
    Map args = buildArguments(arg: {'userId': userId, 'notify': notify}, callback: callback);
    return await _methodChannel.invokeMethod('updateNotifyState', args);
  }

  ///
  /// 是否需要消息通知
  ///
  /// @param userId 用户Id
  /// @return 是否需要通知
  ///
  Future<bool?> notifyForNewMsg(String userId) async {
    Map args = buildArguments(arg: {'userId': userId});
    return await _methodChannel.invokeMethod('notifyForNewMsg', args);
  }

  ///
  /// 静音列表
  ///
  /// @return 静音列表
  ///
  Future<List<User>?> myMuteUserList() async {
    List? list = await _methodChannel.invokeMethod('myMuteUserList');
    return _friendListFromList(list);
  }

  ///
  /// 从云信服务器批量获取用户资料
  ///
  /// @param userIds       用户id列表
  /// @param callback  用户信息回调
  ///
  Future<void> fetchUsers(List<String> userIds, Callback callback) async {
    Map args = buildArguments(arg:  <String, dynamic>{'userIds': userIds}, callback: callback);
    return await _methodChannel.invokeMethod('fetchUsers', args);
  }

  ///
  /// 从本地获取用户资料
  ///
  /// @param  userId 用户id
  /// @return 好友结果
  ///
  Future<User?> friendById(String userId) async {
    Map args = buildArguments(arg: {'userId': userId});
    Map? map = await _methodChannel.invokeMethod('friendById', args);
    return User.fromMap(map);
  }

  ///
  ///  修改自己与目标用户的关系
  ///
  ///  @param user       目标用户
  ///  @param callback   修改结果回调
  ///  @discussion  这个接口提供了备注名的修改以及一些扩展。这些值是基于当前用户和目标用户关系的，
  ///  同一个目标用户的的属性字段会随着登录用户的改变而改变。
  ///
  Future<void> updateFriend(User friend, Callback callback) async {
    Map args = buildArguments(arg: {'friend': friend}, callback: callback);
    return await _methodChannel.invokeMethod('updateFriend', args);
  }

  // 返回用户列表

  void _handleFetchUserInfos(Map map, Callback callback) {
    List? users = map['users'];

    callback(map['code'], _friendListFromList(users));
  }

  List<User>? _friendListFromList(List? list) {
    bool isValid = list is List;
    if (!isValid) {
      return null;
    }


    List<User> userList =  [];
    for(Map map in list as Iterable<Map<dynamic, dynamic>>) {
      if (!(map is Map)) {
        continue;
      }

      User? user = User.fromMap(map);
      if (user == null) {
        continue;
      }

      userList.add(user);
    }
    return userList;
  }
}
