// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

class RTSDataImpl {
  /// 当前的服务器时间， 单位毫秒
  int? timeTag;

  /// 音视频服务器鉴权使用
  int? authUid;

  /// 服务器分配的会话id
  int? channelId;

  /// 分配的通道信息
  List<int>? tunTypes;

  /// 存在ios离线设备的账号类别， 需要发送KEEP_NOTIFY协议
  List<String>? keepCallingAccounts;

  /// 所有参与者的account-uid的映射关系
  Map<String, int>? account2uidMap;

  /// 是否P2P连接
  bool? isP2p;

  ///  码率等客户端配置，json格式，客户端自行解析
  String? clientConfig;

  ///  发起方的账号
  String? account;

  /// 客户端本地产生的会话id
  String? sessionId;

  ///  连接地址信息
  Map<int?, RTSServers?>? serversMap;

  ///  双人白板的通知信息
  RTSNotifyOption? notifyOption;

  ///  多人白板的附加信息字段
  String? extraMessage;

  static RTSDataImpl? _fromMap(Map? map) {
    if (map == null) {
      return null;
    }
    var impl = RTSDataImpl();
    List? tunTypes = map['tunTypes'];
    List keepCallingAccounts = map['keepCallingAccounts'];
    Map serversMap = map['serversMap'];
    impl
      ..timeTag = map['timeTag']
      ..authUid = map['authUid']
      ..channelId = map['channelId']
      ..tunTypes = tunTypes?.map<int>((e) => e as int) as List<int>?
      ..keepCallingAccounts = keepCallingAccounts.map<String>((e) => e as String) as List<String>?
      ..isP2p = map['isP2p']
      ..clientConfig = map['clientConfig']
      ..account = map['account']
      ..serversMap = serversMap.map<int?, RTSServers?>((key, value) {
        return MapEntry<int?, RTSServers?>(int.tryParse(key as String), RTSServers._fromMap(value));
      })
      ..notifyOption = RTSNotifyOption._fromMap(map['notifyOption'])
      ..extraMessage = map['extraMessage'];
    return impl;
  }
}

class RTSTunnelType {
  int audio = 1;
  int data = 2;
}

class RTSServers {
  String? serverConfig;

  int? channelId;

  int? tunType;

  List<String>? turnServersForTcp;

  List<String>? proxyServerList;

  static RTSServers? _fromMap(Map? map) {
    if (map == null) {
      return null;
    }
    RTSServers servers = RTSServers();
    List turnServersForTcp = map['turnServersForTcp'];
    List proxyServerList = map['proxyServerList'];
    servers
      ..serverConfig = map['serverConfig']
      ..channelId = map['channelId']
      ..tunType = map['tunType']
      ..turnServersForTcp = turnServersForTcp.map<String>((e) => e as String) as List<String>?
      ..proxyServerList = proxyServerList.map<String>((e) => e as String) as List<String>?;
    return servers;
  }
}

class RTSNotifyOption {
  bool apnsInuse = true;

  bool apnsBadge = true;

  bool apnsWithPrefix = true;

  String? apnsContent;

  String? extendMessage;

  String? apnsPayload;

  String? apnsSound;

  bool forceKeepCalling = true;

  static RTSNotifyOption? _fromMap(Map? map) {
    if (map == null) {
      return null;
    }
    RTSNotifyOption notifyOption = RTSNotifyOption();
    notifyOption
      ..apnsInuse = map['apnsInuse'] ?? true
      ..apnsBadge = map['apnsBadge'] ?? true
      ..apnsWithPrefix = map['apnsWithPrefix'] ?? true
      ..apnsContent = map['apnsContent']
      ..extendMessage = map['extendMessage']
      ..apnsPayload = map['apnsPayload']
      ..apnsSound = map['apnsSound']
      ..forceKeepCalling = map['forceKeepCalling'] ?? true;
    return notifyOption;
  }
}

class Event {
  RTSEventType? eventType;
  String? sessionId;
  String? channelId;
  String? account;
  int? timeTag;
  String? extra;
  List<int>? tunTypes;

  static Event? _fromMap(Map? map) {
    if (map == null) {
      return null;
    }
    Event event = Event();
    int type = map['eventType'] ?? 0;
    event.eventType =
        (type >= 0 && type < RTSEventType.values.length) ? RTSEventType.values[type] : RTSEventType.undefine;
    event.sessionId = map['sessionId'];
    event.channelId = map['channelId'];
    event.account = map['account'];
    event.timeTag = map['timeTag'];
    event.extra = map['extra'];
    event.tunTypes = map['tunTypes'];
    return event;
  }
}

class RTSEvent {
  String? channelId;
  Event? event;

  static RTSEvent? _fromMap(Map? map) {
    if (map == null) {
      return null;
    }
    RTSEvent event = RTSEvent();
    event.channelId = map['channelId'];
    event.event = Event._fromMap(map['event']);
    return event;
  }
}

/// 会话事件枚举
enum RTSEventType {
  ///
  ///未定义
  ///
  undefine,

  ///
  ///被叫方接受会话
  ///
  calleeAckAgree,

  ///
  ///被叫方拒绝会话
  ///
  calleeAckReject,

  ///
  ///被叫方正在忙
  ///
  calleeAckBusy,

  ///
  ///被叫方同时在线的其他端接受会话
  ///
  calleeOnlineClientAckAgree,

  ///
  ///被叫方同时在线的其他端拒绝会话
  ///
  calleeOnlineClientAckReject,

  ///
  ///对方结束会话
  ///
  peerHangUp,

  ///
  ///会话中收到的控制命令
  ///
  controlNotification,

  ///
  ///更新用户的account和id映射关系
  ///
  notifyAccountIdMapping
}
