// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

/// dart enum must start 0, but define start 1, so unknown is holder
enum ChannelType {
  /// HOLDER
  unknown,

  /// 音频频道
  audio,

  /// 视频频道
  video,

  /// 自定义频道
  custom
}

enum ChannelStatus {
  /// 频道正常
  normal,

  /// 频道无效了 ，列如被关闭了
  invalid,
}

class ChannelBaseInfo {
  final String? channelName;

  final String? channelId;

  ChannelType? type;

  String? channelExt;

  int? createTimestamp;

  int? expireTimestamp;

  String? creatorAccountId;

  ChannelStatus? channelStatus;

  ChannelBaseInfo(
      {required this.channelName,
      required this.channelId,
      this.type,
      this.channelExt,
      this.createTimestamp,
      this.expireTimestamp,
      this.creatorAccountId,
      this.channelStatus});

  static ChannelBaseInfo? _fromMap(Map? map) {
    if (map == null) {
      return null;
    }
    var index = map['type'] ?? 0;
    ChannelType type = index >= 0 && index < ChannelType.values.length
        ? ChannelType.values[index]
        : ChannelType.unknown;
    var index2 = map['channelStatus'] ?? 0;
    ChannelStatus status = index2 >= 0 && index2 < ChannelStatus.values.length
        ? ChannelStatus.values[index2]
        : ChannelStatus.invalid;
    return ChannelBaseInfo(
        channelName: map['channelName'],
        channelId: map['channelId'],
        type: type,
        channelExt: map['channelExt'],
        createTimestamp: map['createTimestamp'],
        expireTimestamp: map['expireTimestamp'],
        creatorAccountId: map['creatorAccountId'],
        channelStatus: status);
  }

  Map<String, dynamic> _toMap() => <String, dynamic>{
        'channelName': channelName,
        'channelId': channelId,
        'type': type!.index,
        'channelExt': channelExt,
        'createTimestamp': createTimestamp,
        'expireTimestamp': expireTimestamp,
        'creatorAccountId': creatorAccountId,
        'channelStatus': channelStatus!.index
      };
}

class MemberInfo {
  String? accountId;

  int? uid;

  int? joinTime;

  int? expireTime;

  MemberInfo({this.accountId, this.uid, this.joinTime, this.expireTime});

  static _fromMap(Map map) {
    return MemberInfo(
        accountId: map['accountId'],
        uid: map['uid'],
        joinTime: map['joinTime'],
        expireTime: map['expireTime']);
  }
}

class ChannelFullInfo {
  ChannelBaseInfo? channelBaseInfo;

  List<MemberInfo>? members;

  ChannelFullInfo({required this.channelBaseInfo, this.members});

  static ChannelFullInfo? _fromMap(Map? arg) {
    if (arg == null) {
      return null;
    }
    List? list = arg['members'] ?? null;
    List<MemberInfo>? members = list != null
        ? list.map<MemberInfo>((i) {
            if (i is String) {
              Map value = json.decode(i);
              return MemberInfo._fromMap(value);
            }
            return MemberInfo._fromMap(i);
          }).toList()
        : null;
    return ChannelFullInfo(
        channelBaseInfo: ChannelBaseInfo._fromMap(arg['channelBaseInfo']),
        members: members);
  }
}

class SignallingPushConfig {
  bool? needPush;

  String? pushTitle;

  String? pushContent;

  /// key - value {json style}
  String? pushPayload;

  bool? needBadge;

  SignallingPushConfig(
      {this.needPush,
      this.pushTitle,
      this.pushContent,
      this.pushPayload,
      this.needBadge});

  Map<String, dynamic> _toMap() => {
        'needPush': needPush,
        'pushTitle': pushTitle,
        'pushContent': pushContent,
        'pushPayload': pushPayload,
        'needBadge': needBadge,
      };

  static SignallingPushConfig _fromMap(Map map) {
    return SignallingPushConfig(
        needPush: map['needPush'],
        pushTitle: map['pushTitle'],
        pushContent: map['pushContent'],
        pushPayload: map['pushPayload'],
        needBadge: map['needBadge']);
  }
}

enum SignallingEventType {
  /// 未知事件，占位符
  unknown,

  /// 频道关闭事件
  close,

  /// 有人加入频道事件
  join,

  /// 被邀请事件
  invite,

  /// 取消邀请事件
  cancelInvite,

  /// 对方拒绝邀请事件
  reject,

  /// 对方接受邀请事件
  accept,

  /// 有用户离开频道事件
  leave,

  /// 自定义控制命令事件
  control
}

enum InviteAckStatus {
  /// holder
  unknown,

  /// 拒绝邀请
  reject,

  /// 接受邀请
  accept,
}

class SignallingEvent {
  ///获取频道基础信息
  ChannelBaseInfo? baseInfo;

  ///
  /// 通知来自于谁（由谁触发者的通知）
  /// 例如：
  /// 如果是被邀请事件{@link InvitedEvent} ，那么from 就是主动邀请者
  /// 如果是邀请应答事件{@link InviteAckEvent} ，那么from 就是响应的邀请的人
  /// 注意：如果是多端同步的通知，那么from一般指自己
  ///
  String? fromAccountId;

  /// 通知触发者附加的自定义信息
  String? customInfo;

  /// 通知事件类型
  SignallingEventType? type;

  /// 通知产生的时间（指对方发送通知的时间）
  int? time;

  SignallingEvent(
      {required this.baseInfo,
      this.fromAccountId,
      this.customInfo,
      this.type,
      this.time});

  static SignallingEvent _fromMap(Map map, SignallingEventType type) {
    return SignallingEvent(
        baseInfo: ChannelBaseInfo._fromMap(map['baseInfo']),
        fromAccountId: map['fromAccountId'],
        customInfo: map['customInfo'],
        type: type,
        time: map['time']);
  }
}

/// 有用户加入频道通知事件
class UserJoinEvent extends SignallingEvent {
  MemberInfo? memberInfo;

  UserJoinEvent(
      {baseInfo, fromAccountId, customInfo, type, time, this.memberInfo})
      : super(
            baseInfo: baseInfo,
            fromAccountId: fromAccountId,
            customInfo: customInfo,
            type: type,
            time: time);

  static UserJoinEvent _fromMap(Map map, SignallingEventType type) {
    return UserJoinEvent(
        baseInfo: ChannelBaseInfo._fromMap(map['baseInfo']),
        fromAccountId: map['fromAccountId'],
        customInfo: map['customInfo'],
        type: type,
        time: map['time'],
        memberInfo: MemberInfo._fromMap(map['memberInfo']));
  }
}

/// 被邀请通知事件
class InviteEvent extends SignallingEvent {
  String? toAccountId;
  String? requestId;
  SignallingPushConfig? pushConfig;

  InviteEvent(
      {baseInfo,
      fromAccountId,
      customInfo,
      type,
      time,
      this.toAccountId,
      this.requestId,
      this.pushConfig})
      : super(
            baseInfo: baseInfo,
            fromAccountId: fromAccountId,
            customInfo: customInfo,
            type: type,
            time: time);

  static InviteEvent _fromMap(Map map, SignallingEventType type) {
    return InviteEvent(
        baseInfo: ChannelBaseInfo._fromMap(map['baseInfo']),
        fromAccountId: map['fromAccountId'],
        customInfo: map['customInfo'],
        type: type,
        time: map['time'],
        toAccountId: map['toAccountId'],
        requestId: map['requestId'],
        pushConfig: SignallingPushConfig._fromMap(map['pushConfig']));
  }
}

/// 被取消邀请通知事件
class CanceledInviteEvent extends SignallingEvent {
  String? toAccount;
  String? requestId;

  CanceledInviteEvent({
    baseInfo,
    fromAccountId,
    customInfo,
    type,
    time,
    this.toAccount,
    this.requestId,
  }) : super(
            baseInfo: baseInfo,
            fromAccountId: fromAccountId,
            customInfo: customInfo,
            type: type,
            time: time);

  static CanceledInviteEvent _fromMap(Map map, SignallingEventType type) {
    return CanceledInviteEvent(
      baseInfo: ChannelBaseInfo._fromMap(map['baseInfo']),
      fromAccountId: map['fromAccountId'],
      customInfo: map['customInfo'],
      type: type,
      time: map['time'],
      toAccount: map['toAccount'],
      requestId: map['requestId'],
    );
  }
}

/// 对方应答（接受/拒绝）邀请通知事件
class InviteAckEvent extends SignallingEvent {
  String? toAccountId;
  String? requestId;
  InviteAckStatus? status;

  InviteAckEvent(
      {baseInfo,
      fromAccountId,
      customInfo,
      type,
      time,
      this.toAccountId,
      this.requestId,
      this.status})
      : super(
            baseInfo: baseInfo,
            fromAccountId: fromAccountId,
            customInfo: customInfo,
            type: type,
            time: time);

  static InviteAckEvent _fromMap(Map map, SignallingEventType type) {
    var index = map['status'];
    InviteAckStatus status = index >= 0 && index < InviteAckStatus.values.length
        ? InviteAckStatus.values[index]
        : InviteAckStatus.unknown;
    return InviteAckEvent(
        baseInfo: ChannelBaseInfo._fromMap(map['baseInfo']),
        fromAccountId: map['fromAccountId'],
        customInfo: map['customInfo'],
        type: type,
        time: map['time'],
        toAccountId: map['toAccountId'],
        requestId: map['requestId'],
        status: status);
  }

  static InviteAckEvent __fromMap(Map map) {
    int index = map['type'] ?? 0;
    SignallingEventType type =
        index >= 0 && index < SignallingEventType.values.length
            ? SignallingEventType.values[index]
            : SignallingEventType.unknown;
    return _fromMap(map, type);
  }
}

class EventFactory {
  static SignallingEvent _fromMap(Map map) {
    int index = map['type'] ?? 0;
    SignallingEventType type =
        index >= 0 && index < SignallingEventType.values.length
            ? SignallingEventType.values[index]
            : SignallingEventType.unknown;

    switch (type) {
      case SignallingEventType.join:
        return UserJoinEvent._fromMap(map, type);
      case SignallingEventType.invite:
        return InviteEvent._fromMap(map, type);
      case SignallingEventType.cancelInvite:
        return CanceledInviteEvent._fromMap(map, type);
      case SignallingEventType.reject:
      case SignallingEventType.accept:
        return InviteAckEvent._fromMap(map, type);
      default:
        return SignallingEvent._fromMap(map, type);
    }
  }
}
