// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

class SignallingService extends Service {
  SignallingService(MethodChannel methodChannel, handlerMap) : super(methodChannel, handlerMap);

  @override
  String _getModule() {
    return "SignallingService";
  }

  ///
  ///创建频道<br/>
  ///同一时刻频道名互斥，不能重复创建，但如果频道名缺省，服务器会自动分配频道id。<br/><br/>
  ///错误码如下：<br/>
  ///10405：频道已存在
  /// 回调返回： code = 200 为成功 data: ChannelBaseInfo
  ///
  Future<void> create(
      {required ChannelType type,
      required String channelName,
      String? channelExt,
      required Callback<ChannelBaseInfo> callback}) async {
    Map map = buildArguments(callback: callback);
    map['type'] = type.index;
    map['channelName'] = channelName;
    map['channelExt'] = channelExt;
    return await _methodChannel.invokeMethod('create', map);
  }

  ///
  /// 关闭频道<br/><br/>
  /// 错误码如下：<br/>
  /// 10406：不在频道内
  ///
  /// @param channelId      对应频道id
  /// @param offlineEnabled 通知事件是否存离线
  /// @param customInfo     操作者附加的自定义信息，透传给其他人，可缺省
  /// 回调返回： code = 200 为成功
  ///
  Future<void> close(
      {required String channelId, bool? offlineEnable, String? customInfo, required Callback callback}) async {
    Map map = buildArguments(callback: callback);
    map['channelId'] = channelId;
    map['offlineEnable'] = offlineEnable ?? false;
    map['customInfo'] = customInfo;
    return await _methodChannel.invokeMethod('close', map);
  }

  ///
  /// 加入频道<br/><br/>
  /// 错误码如下：<br/>
  /// 10407：已经频道内
  /// 10417：uid冲突
  /// 10419：频道人数超限
  /// 10420：自己的其他端已经在频道内
  ///
  /// @param channelId     对应频道id
  /// @param selfUid       自己在频道中对应的uid，可选，大于零有效，无效时服务器会分配随机唯一的uid， 也可以自己生成，但要保证唯一性
  /// @param customInfo    操作者附加的自定义信息，透传给其他人，可缺省
  /// @param offlineEnable 通知事件是否存离线
  /// 回调返回： code = 200 为成功
  ///
  Future<void> join(
      {required String channelId,
      int? selfUid,
      String? customInfo,
      bool? offlineEnable,
      required Callback<ChannelFullInfo> callback}) async {
    Map map = buildArguments(callback: callback);
    map['channelId'] = channelId;
    map['selfUid'] = selfUid ?? 0;
    map['customInfo'] = customInfo;
    map['offlineEnable'] = offlineEnable ?? false;
    return await _methodChannel.invokeMethod('join', map);
  }

  ///
  /// 离开频道
  /// 错误码如下：
  /// 10406：不在频道内
  ///
  /// @param channelId     对应频道id
  /// @param offlineEnable 通知事件是否存离线
  /// @param customInfo    操作者附加的自定义信息，透传给其他人，可缺省
  /// 回调返回： code = 200 为成功
  ///
  Future<void> leave(
      {required String channelId, bool? offlineEnable, String? customInfo, required Callback callback}) async {
    Map map = buildArguments(callback: callback);
    map['channelId'] = channelId;
    map['customInfo'] = customInfo;
    map['offlineEnable'] = offlineEnable ?? false;
    return await _methodChannel.invokeMethod('leave', map);
  }

  ///
  /// 邀请他人加入频道
  /// 该接口用于邀请对方加入频道，邀请者必须是创建者或者是频道中成员。如果需要对离线成员邀请，可以打开离线邀请开关并填写推送信息，被邀请者上线后通过离线通知接收到该邀请。
  /// 错误码如下：
  /// 10201：已经成功发出邀请但是对方不在线（推送可达，但是离线）
  /// 10202：已经成功发出邀请但是对方推送不可达
  /// 10404：频道不存在
  /// 10406：自己不在频道内（仅对于普通用户，频道创建者不在频道内也可以邀请别人）
  /// 10407：对方已经频道内
  /// 10419：频道人数超限
  /// 回调返回： code = 200 为成功
  ///
  Future<void> invite(
      {required String channelId,
      required String accountId,
      required String requestId,
      String? customInfo,
      required SignallingPushConfig pushConfig,
      bool? offlineEnable,
      required Callback callback}) async {
    Map map = buildArguments(callback: callback);
    map['channelId'] = channelId;
    map['accountId'] = accountId;
    map['requestId'] = requestId;
    map['customInfo'] = customInfo;
    map['pushConfig'] = pushConfig._toMap();
    map['offlineEnable'] = offlineEnable ?? false;
    return await _methodChannel.invokeMethod('invite', map);
  }

  ///
  /// 邀取消邀请
  /// 错误码如下：
  /// 10404：频道不存在
  /// 10408：邀请不存在或已过期
  /// 10409：邀请已经拒绝
  /// 10410：邀请已经接受
  /// 回调返回： code = 200 为成功
  ///
  Future<void> cancelInvite(
      {required String channelId,
      required String accountId,
      required String requestId,
      String? customInfo,
      bool? offlineEnable,
      required Callback callback}) async {
    Map map = buildArguments(callback: callback);
    map['channelId'] = channelId;
    map['accountId'] = accountId;
    map['requestId'] = requestId;
    map['customInfo'] = customInfo;
    map['offlineEnable'] = offlineEnable ?? false;
    return await _methodChannel.invokeMethod('cancelInvite', map);
  }

  ///
  /// 拒绝对方邀请
  /// 错误码如下：
  /// 10201：已经成功拒绝邀请但是对方不在线
  /// 10404：频道不存在
  /// 10408：邀请不存在或已过期
  /// 10409：邀请已经拒绝
  /// 10410：邀请已经接受
  /// 回调返回： code = 200 为成功
  ///
  Future<void> rejectInvite(
      {required String channelId,
      required String accountId,
      required String requestId,
      String? customInfo,
      bool? offlineEnable,
      required Callback callback}) async {
    Map map = buildArguments(callback: callback);
    map['channelId'] = channelId;
    map['accountId'] = accountId;
    map['requestId'] = requestId;
    map['customInfo'] = customInfo;
    map['offlineEnable'] = offlineEnable ?? false;
    return await _methodChannel.invokeMethod('rejectInvite', map);
  }

  ///
  /// 接受对方邀请，但并不代表加入了频道，如果想在接受邀请时自动加入频道, acceptInviteAndJoin
  /// 错误码如下：
  /// 10201：已经成功拒绝邀请但是对方不在线
  /// 10404：频道不存在
  /// 10408：邀请不存在或已过期
  /// 10409：邀请已经拒绝
  /// 10410：邀请已经接受
  /// 回调返回： code = 200 为成功
  ///
  Future<void> acceptInvite(
      {required String channelId,
      required String accountId,
      required String requestId,
      String? customInfo,
      bool? offlineEnable,
      required Callback callback}) async {
    Map map = buildArguments(callback: callback);
    map['channelId'] = channelId;
    map['accountId'] = accountId;
    map['requestId'] = requestId;
    map['customInfo'] = customInfo;
    map['offlineEnable'] = offlineEnable ?? false;
    return await _methodChannel.invokeMethod('acceptInvite', map);
  }

  ///
  /// 接受对方邀请并自动加入频道, 组合接口
  /// 错误码如下：
  /// 10420：自己的其他端已经在频道内
  /// 10201：已经成功拒绝邀请但是对方不在线
  /// 10404：频道不存在
  /// 10407：已经频道内
  /// 10408：邀请不存在或已过期
  /// 10409：邀请已经拒绝
  /// 10410：邀请已经接受
  /// 10417：uid冲突
  /// 10419：频道人数超限
  /// 回调返回： code = 200 为成功 data: ChannelFullInfo
  ///
  /// @param selfUid      自己在频道中对应的uid，大于零有效，无效时服务器会分配随机唯一的uid， 也可以自己生成，但要保证唯一性
  ///
  Future<void> acceptInviteAndJoin(
      {required String channelId,
      required String accountId,
      required String requestId,
      String? customInfo,
      bool? offlineEnable,
      BigInt? selfUid,
      required Callback<ChannelFullInfo> callback}) async {
    Map map = buildArguments(callback: callback);
    map['channelId'] = channelId;
    map['accountId'] = accountId;
    map['requestId'] = requestId;
    map['customInfo'] = customInfo;
    map['offlineEnabled'] = offlineEnable ?? false;
    map['selfUid'] = selfUid ?? 0;
    return await _methodChannel.invokeMethod('acceptInviteAndJoin', map);
  }

  ///
  /// 该接口用于在频道中透传一些自定义指令，协助频道管理。该接口允许非频道内成员调用，但接收者必须是频道内成员或创建者
  /// accountId 不传表示通知所有人
  /// 错误码如下：
  /// 10201：已经成功发出命令但是对方不在线（推送可达，但是离线）
  /// 10404：频道不存在
  /// 10406：不在频道内（自己或者对方）
  /// 回调返回： code = 200 为成功
  ///
  Future<void> sendControl(
      {required String channelId, String? accountId, String? customInfo, required Callback<void> callback}) async {
    Map map = buildArguments(callback: callback);
    map['channelId'] = channelId;
    map['accountId'] = accountId;
    map['customInfo'] = customInfo;
    return await _methodChannel.invokeMethod('sendControl', map);
  }

  ///
  /// 直接呼叫 ， 用于用户新开一个频道并邀请对方加入频道。该接口为组合接口，等同于用户先创建频道，然后加入频道并邀请对方
  /// 错误码如下：
  /// 10201：已经成功发出命令但是对方不在线（推送可达，但是离线）
  /// 10202: 已经成功发出邀请但是对方推送不可达
  /// 10405：频道已存在
  /// 回调返回： code = 200 为成功 data: ChannelFullInfo
  ///
  Future<void> call(
      {required ChannelType type,
      required String accountId,
      required String requestId,
      String? channelName,
      String? channelExt,
      BigInt? selfUid,
      bool? offlineEnable,
      String? customInfo,
      required SignallingPushConfig pushConfig,
      required Callback<ChannelFullInfo> callback}) async {
    Map map = buildArguments(callback: callback);
    map['type'] = type.index;
    map['accountId'] = accountId;
    map['requestId'] = requestId;
    map['channelName'] = channelName;
    map['channelExt'] = channelExt;
    map['selfUid'] = selfUid ?? 0;
    map['offlineEnable'] = offlineEnable ?? false;
    map['customInfo'] = customInfo;
    map['pushConfig'] = pushConfig._toMap();
    return await _methodChannel.invokeMethod('call', map);
  }

  ///
  ///根据channelName反查channelInfo
  ///错误码如下：
  ///10404:频道不存在
  ///回调返回： code = 200 为成功 data: ChannelBaseInfo
  ///
  Future<void> queryChannelInfo({required String channelName, required Callback<ChannelFullInfo> callback}) async {
    Map map = buildArguments(callback: callback);
    map['channelName'] = channelName;
    return await _methodChannel.invokeMethod('queryChannelInfo', map);
  }

  @override
  Future? _handlerMethod(String method, int code, Map arg, dynamic callback) {
    switch (method) {
      case 'onCreate':
        ChannelBaseInfo? baseInfo = ChannelBaseInfo._fromMap(arg['channelBaseInfo']);
        callback(code, baseInfo);
        break;
      case 'onJoin':
      case 'onAcceptInviteAndJoin':
      case 'onCall':
      case 'onQueryChannelInfo':
        ChannelFullInfo? fullInfo = ChannelFullInfo._fromMap(arg['channelFullInfo']);
        callback(code, fullInfo);
        break;
      default:
        callback(code, null);
    }
    return null;
  }
}
